@@ -1,12 +1,12 @@
 /*
  * MUSCLE SmartCard Development ( http://www.linuxnet.com )
  *
- * Copyright (C) 2000
+ * Copyright (C) 2000-2002
  *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2004-2008
+ * Copyright (C) 2002-2009
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: eventhandler.c 3305 2009-02-06 08:54:05Z rousseau $
+ * $Id: eventhandler.c 4912 2010-05-03 09:30:25Z rousseau $
  */
 
 /**
@@ -22,12 +22,12 @@
 #include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
+#include <pthread.h>
 
 #include "misc.h"
 #include "pcscd.h"
 #include "ifdhandler.h"
 #include "debuglog.h"
-#include "thread_generic.h"
 #include "readerfactory.h"
 #include "eventhandler.h"
 #include "dyn_generic.h"
@@ -36,71 +36,115 @@
 #include "prothandler.h"
 #include "strlcpycat.h"
 #include "utils.h"
+#include "winscard_svc.h"
+#include "simclist.h"
 
-static PREADER_STATE readerStates[PCSCLITE_MAX_READERS_CONTEXTS];
+READER_STATE readerStates[PCSCLITE_MAX_READERS_CONTEXTS];
+static list_t ClientsWaitingForEvent;	/**< list of client file descriptors */
+pthread_mutex_t ClientsWaitingForEvent_lock;	/**< lock for the above list */
 
-static void EHStatusHandlerThread(PREADER_CONTEXT);
+static void EHStatusHandlerThread(READER_CONTEXT *);
+
+LONG EHRegisterClientForEvent(int32_t filedes)
+{
+	(void)pthread_mutex_lock(&ClientsWaitingForEvent_lock);
+
+	(void)list_append(&ClientsWaitingForEvent, &filedes);
+	
+	(void)pthread_mutex_unlock(&ClientsWaitingForEvent_lock);
+
+	return SCARD_S_SUCCESS;
+} /* EHRegisterClientForEvent */
+
+/**
+ * Try to unregisted a client
+ * If no client is found then do no log an error
+ */
+LONG EHTryToUnregisterClientForEvent(int32_t filedes)
+{
+	LONG rv = SCARD_S_SUCCESS;
+	int ret;
+
+	(void)pthread_mutex_lock(&ClientsWaitingForEvent_lock);
+
+	ret = list_delete(&ClientsWaitingForEvent, &filedes);
+	
+	(void)pthread_mutex_unlock(&ClientsWaitingForEvent_lock);
+	
+	if (ret < 0)
+		rv = SCARD_F_INTERNAL_ERROR;
+
+	return rv;
+} /* EHTryToUnregisterClientForEvent */
+
+/**
+ * Unregister a client and log an error if the client is not found
+ */
+LONG EHUnregisterClientForEvent(int32_t filedes)
+{
+	LONG rv = EHTryToUnregisterClientForEvent(filedes);
+	
+	if (rv < 0)
+		Log2(PCSC_LOG_ERROR, "Can't remove client: %d", filedes);
+
+	return rv;
+} /* EHUnregisterClientForEvent */
+
+/**
+ * Sends an asynchronous event to any waiting client
+ */
+LONG EHSignalEventToClients(void)
+{
+	LONG rv = SCARD_S_SUCCESS;
+	int32_t filedes;
+
+	(void)pthread_mutex_lock(&ClientsWaitingForEvent_lock);
+
+	(void)list_iterator_start(&ClientsWaitingForEvent);
+	while (list_iterator_hasnext(&ClientsWaitingForEvent))
+	{
+        filedes = *(int32_t *)list_iterator_next(&ClientsWaitingForEvent);
+		rv = MSGSignalClient(filedes, SCARD_S_SUCCESS);
+	}
+	(void)list_iterator_stop(&ClientsWaitingForEvent);
+
+	(void)list_clear(&ClientsWaitingForEvent);
+
+	(void)pthread_mutex_unlock(&ClientsWaitingForEvent_lock);
+
+	return rv;
+} /* EHSignalEventToClients */
 
 LONG EHInitializeEventStructures(void)
 {
-	int fd, i, pageSize;
-	int mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+	int i;
 
-	fd = 0;
-	i = 0;
-	pageSize = 0;
-
-	(void)SYS_RemoveFile(PCSCLITE_PUBSHM_FILE);
-
-	fd = SYS_OpenFile(PCSCLITE_PUBSHM_FILE, O_RDWR | O_CREAT, mode);
-	if (fd < 0)
-	{
-		Log3(PCSC_LOG_CRITICAL, "Cannot create public shared file %s: %s",
-			PCSCLITE_PUBSHM_FILE, strerror(errno));
-		exit(1);
-	}
-
-	/* set correct mode even is umask is too restictive */
-	(void)SYS_Chmod(PCSCLITE_PUBSHM_FILE, mode);
-
-	pageSize = SYS_GetPageSize();
-
-	/*
-	 * Jump to end of file space and allocate zero's
-	 */
-	(void)SYS_SeekFile(fd, pageSize * PCSCLITE_MAX_READERS_CONTEXTS);
-	(void)SYS_WriteFile(fd, "", 1);
-
-	/*
-	 * Allocate each reader structure
-	 */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		readerStates[i] = (PREADER_STATE)
-			SYS_MemoryMap(sizeof(READER_STATE), fd, (i * pageSize));
-		if (readerStates[i] == MAP_FAILED)
-		{
-			Log3(PCSC_LOG_CRITICAL, "Cannot memory map public shared file %s: %s",
-				PCSCLITE_PUBSHM_FILE, strerror(errno));
-			exit(1);
-		}
-
-		/*
-		 * Zero out each value in the struct
-		 */
-		memset((readerStates[i])->readerName, 0, MAX_READERNAME);
-		memset((readerStates[i])->cardAtr, 0, MAX_ATR_SIZE);
-		(readerStates[i])->readerID = 0;
-		(readerStates[i])->readerState = 0;
-		(readerStates[i])->readerSharing = 0;
-		(readerStates[i])->cardAtrLength = 0;
-		(readerStates[i])->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
+		/* Zero out each value in the struct */
+		memset(readerStates[i].readerName, 0, MAX_READERNAME);
+		memset(readerStates[i].cardAtr, 0, MAX_ATR_SIZE);
+		readerStates[i].readerID = 0;
+		readerStates[i].readerState = 0;
+		readerStates[i].readerSharing = 0;
+		readerStates[i].cardAtrLength = 0;
+		readerStates[i].cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 	}
 
+	(void)list_init(&ClientsWaitingForEvent);
+
+	/* request to store copies, and provide the metric function */
+    (void)list_attributes_copy(&ClientsWaitingForEvent, list_meter_int32_t, 1);
+
+	/* setting the comparator, so the list can sort, find the min, max etc */
+    (void)list_attributes_comparator(&ClientsWaitingForEvent, list_comparator_int32_t);
+
+	(void)pthread_mutex_init(&ClientsWaitingForEvent_lock, NULL);
+
 	return SCARD_S_SUCCESS;
 }
 
-LONG EHDestroyEventHandler(PREADER_CONTEXT rContext)
+LONG EHDestroyEventHandler(READER_CONTEXT * rContext)
 {
 	int rv;
 	DWORD dwGetSize;
@@ -121,7 +165,7 @@ LONG EHDestroyEventHandler(PREADER_CONTEXT rContext)
 	/*
 	 * Set the thread to 0 to exit thread
 	 */
-	rContext->dwLockId = 0xFFFF;
+	rContext->hLockId = 0xFFFF;
 
 	Log1(PCSC_LOG_INFO, "Stomping thread.");
 
@@ -130,18 +174,20 @@ LONG EHDestroyEventHandler(PREADER_CONTEXT rContext)
 	rv = IFDGetCapabilities(rContext, TAG_IFD_POLLING_THREAD_KILLABLE,
 		&dwGetSize, ucGetData);
 
+#ifdef HAVE_PTHREAD_CANCEL
 	if ((IFD_SUCCESS == rv) && (1 == dwGetSize) && ucGetData[0])
 	{
 		Log1(PCSC_LOG_INFO, "Killing polling thread");
-		(void)SYS_ThreadCancel(rContext->pthThread);
+		(void)pthread_cancel(rContext->pthThread);
 	}
 	else
+#endif
 		Log1(PCSC_LOG_INFO, "Waiting polling thread");
 
 	/* wait for the thread to finish */
-	rv = SYS_ThreadJoin(rContext->pthThread, NULL);
+	rv = pthread_join(rContext->pthThread, NULL);
 	if (rv)
-		Log2(PCSC_LOG_ERROR, "SYS_ThreadJoin failed: %s", strerror(rv));
+		Log2(PCSC_LOG_ERROR, "pthread_join failed: %s", strerror(rv));
 
 	/*
 	 * Zero out the public status struct to allow it to be recycled and
@@ -165,7 +211,7 @@ LONG EHDestroyEventHandler(PREADER_CONTEXT rContext)
 	return SCARD_S_SUCCESS;
 }
 
-LONG EHSpawnEventHandler(PREADER_CONTEXT rContext,
+LONG EHSpawnEventHandler(READER_CONTEXT * rContext,
 	RESPONSECODE (*card_event)(DWORD))
 {
 	LONG rv;
@@ -186,7 +232,7 @@ LONG EHSpawnEventHandler(PREADER_CONTEXT rContext,
 	 */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
-		if ((readerStates[i])->readerID == 0)
+		if (readerStates[i].readerID == 0)
 			break;
 	}
 
@@ -196,22 +242,22 @@ LONG EHSpawnEventHandler(PREADER_CONTEXT rContext,
 	/*
 	 * Set all the attributes to this reader
 	 */
-	rContext->readerState = readerStates[i];
+	rContext->readerState = &readerStates[i];
 	(void)strlcpy(rContext->readerState->readerName, rContext->lpcReader,
 		sizeof(rContext->readerState->readerName));
 	memcpy(rContext->readerState->cardAtr, ucAtr, dwAtrLen);
 	rContext->readerState->readerID = i + 100;
 	rContext->readerState->readerState = dwStatus;
-	rContext->readerState->readerSharing = rContext->dwContexts;
+	rContext->readerState->readerSharing = rContext->contexts;
 	rContext->readerState->cardAtrLength = dwAtrLen;
 	rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 
 	rContext->pthCardEvent = card_event;
-	rv = SYS_ThreadCreate(&rContext->pthThread, 0,
+	rv = ThreadCreate(&rContext->pthThread, 0,
 		(PCSCLITE_THREAD_FUNCTION( ))EHStatusHandlerThread, (LPVOID) rContext);
 	if (rv)
 	{
-		Log2(PCSC_LOG_ERROR, "SYS_ThreadCreate failed: %s", strerror(rv));
+		Log2(PCSC_LOG_ERROR, "ThreadCreate failed: %s", strerror(rv));
 		return SCARD_E_NO_MEMORY;
 	}
 	else
@@ -228,11 +274,12 @@ static void incrementEventCounter(struct pubReaderStatesList *readerState)
 		+ (counter << 16);
 }
 
-static void EHStatusHandlerThread(PREADER_CONTEXT rContext)
+static void EHStatusHandlerThread(READER_CONTEXT * rContext)
 {
 	LONG rv;
 	LPCSTR lpcReader;
-	DWORD dwStatus, dwReaderSharing;
+	DWORD dwStatus;
+	int32_t readerSharing;
 	DWORD dwCurrentState;
 	DWORD dwAtrLen;
 
@@ -240,7 +287,7 @@ static void EHStatusHandlerThread(PREADER_CONTEXT rContext)
 	 * Zero out everything
 	 */
 	dwStatus = 0;
-	dwReaderSharing = 0;
+	readerSharing = 0;
 	dwCurrentState = 0;
 
 	lpcReader = rContext->lpcReader;
@@ -313,10 +360,10 @@ static void EHStatusHandlerThread(PREADER_CONTEXT rContext)
 	 * Set all the public attributes to this reader
 	 */
 	rContext->readerState->readerState = dwStatus;
-	rContext->readerState->readerSharing = dwReaderSharing =
-		rContext->dwContexts;
+	rContext->readerState->readerSharing = readerSharing =
+		rContext->contexts;
 
-	(void)StatSynchronize(rContext->readerState);
+	(void)EHSignalEventToClients();
 
 	while (1)
 	{
@@ -347,7 +394,7 @@ static void EHStatusHandlerThread(PREADER_CONTEXT rContext)
 
 			dwCurrentState = SCARD_UNKNOWN;
 
-			(void)StatSynchronize(rContext->readerState);
+			(void)EHSignalEventToClients();
 		}
 
 		if (dwStatus & SCARD_ABSENT)
@@ -377,7 +424,7 @@ static void EHStatusHandlerThread(PREADER_CONTEXT rContext)
 
 				incrementEventCounter(rContext->readerState);
 
-				(void)StatSynchronize(rContext->readerState);
+				(void)EHSignalEventToClients();
 			}
 
 		}
@@ -424,10 +471,10 @@ static void EHStatusHandlerThread(PREADER_CONTEXT rContext)
 
 				incrementEventCounter(rContext->readerState);
 
-				(void)StatSynchronize(rContext->readerState);
-
 				Log2(PCSC_LOG_INFO, "Card inserted into %s", lpcReader);
 
+				(void)EHSignalEventToClients();
+
 				if (rv == IFD_SUCCESS)
 				{
 					if (rContext->readerState->cardAtrLength > 0)
@@ -447,33 +494,33 @@ static void EHStatusHandlerThread(PREADER_CONTEXT rContext)
 		/*
 		 * Sharing may change w/o an event pass it on
 		 */
-		if (dwReaderSharing != rContext->dwContexts)
+		if (readerSharing != rContext->contexts)
 		{
-			dwReaderSharing = rContext->dwContexts;
-			rContext->readerState->readerSharing = dwReaderSharing;
-			(void)StatSynchronize(rContext->readerState);
+			readerSharing = rContext->contexts;
+			rContext->readerState->readerSharing = readerSharing;
+			(void)EHSignalEventToClients();
 		}
 
 		if (rContext->pthCardEvent)
 		{
 			int ret;
 
-			ret = rContext->pthCardEvent(rContext->dwSlot);
+			ret = rContext->pthCardEvent(rContext->slot);
 			if (IFD_NO_SUCH_DEVICE == ret)
 				(void)SYS_USleep(PCSCLITE_STATUS_POLL_RATE);
 		}
 		else
 			(void)SYS_USleep(PCSCLITE_STATUS_POLL_RATE);
 
-		if (rContext->dwLockId == 0xFFFF)
+		if (rContext->hLockId == 0xFFFF)
 		{
 			/*
 			 * Exit and notify the caller
 			 */
-			(void)StatSynchronize(rContext->readerState);
+			(void)EHSignalEventToClients();
 			Log1(PCSC_LOG_INFO, "Die");
-			rContext->dwLockId = 0;
-			(void)SYS_ThreadExit(NULL);
+			rContext->hLockId = 0;
+			(void)pthread_exit(NULL);
 		}
 	}
 }
