@@ -1,10 +1,10 @@
 /*
  * MUSCLE SmartCard Development ( http://www.linuxnet.com )
  *
- * Copyright (C) 2008
+ * Copyright (C) 2008-2009
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: hotplug_libhal.c 4249 2009-06-05 08:41:59Z rousseau $
+ * $Id: hotplug_libhal.c 4901 2010-04-29 07:26:36Z rousseau $
  */
 
 /**
@@ -13,13 +13,14 @@
  */
 
 #include "config.h"
-#ifdef HAVE_LIBHAL
+#if defined(HAVE_LIBHAL) && defined(USE_USB)
 
 #include <string.h>
 #include <stdio.h>
 #include <dirent.h>
 #include <stdlib.h>
 #include <libhal.h>
+#include <pthread.h>
 
 #include "misc.h"
 #include "wintypes.h"
@@ -29,20 +30,21 @@
 #include "readerfactory.h"
 #include "sys_generic.h"
 #include "hotplug.h"
-#include "thread_generic.h"
 #include "utils.h"
+#include "strlcpycat.h"
 
 #undef DEBUG_HOTPLUG
 #define ADD_SERIAL_NUMBER
+#define ADD_INTERFACE_NAME
 
 #define FALSE			0
 #define TRUE			1
 
 #define UDI_BASE "/org/freedesktop/Hal/devices/"
 
-extern PCSCLITE_MUTEX usbNotifierMutex;
+pthread_mutex_t usbNotifierMutex;
 
-static PCSCLITE_THREAD_T usbNotifyThread;
+static pthread_t usbNotifyThread;
 static int driverSize = -1;
 static char AraKiriHotPlug = FALSE;
 
@@ -84,10 +86,12 @@ static void HPEstablishUSBNotifications(void);
  *
  * @param  udi                 Universal Device Id
  */
+#ifndef NO_LOG
 static const char *short_name(const char *udi)
 {
 	return &udi[sizeof(UDI_BASE) - 1];
 } /* short_name */
+#endif
 
 
 static LONG HPReadBundleValues(void)
@@ -347,6 +351,8 @@ static void HPAddDevice(LibHalContext *ctx, const char *udi)
 	int i;
 	char deviceName[MAX_DEVICENAME];
 	struct _driverTracker *driver;
+	char *sSerialNumber = NULL, *sInterfaceName = NULL;
+	char fullname[MAX_READERNAME];
 	LONG ret;
 
 	driver = get_driver(ctx, udi);
@@ -368,7 +374,7 @@ static void HPAddDevice(LibHalContext *ctx, const char *udi)
 	/* wait until the device is visible by libusb/etc.  */
 	(void)SYS_Sleep(1);
 
-	(void)SYS_MutexLock(&usbNotifierMutex);
+	(void)pthread_mutex_lock(&usbNotifierMutex);
 
 	/* find a free entry */
 	for (i=0; i<PCSCLITE_MAX_READERS_CONTEXTS; i++)
@@ -381,32 +387,50 @@ static void HPAddDevice(LibHalContext *ctx, const char *udi)
 	{
 		Log2(PCSC_LOG_ERROR,
 			"Not enough reader entries. Already found %d readers", i);
-		(void)SYS_MutexUnLock(&usbNotifierMutex);
+		(void)pthread_mutex_unlock(&usbNotifierMutex);
 		return;
 	}
 
 	readerTracker[i].udi = strdup(udi);
 
+#ifdef ADD_INTERFACE_NAME
+	if (libhal_device_property_exists(ctx, udi, "usb.interface.description", NULL))
+		sInterfaceName = libhal_device_get_property_string(ctx, udi,
+			"usb.interface.description", NULL);
+#endif
+
 #ifdef ADD_SERIAL_NUMBER
 	if (libhal_device_property_exists(ctx, udi, "usb.serial", NULL))
-	{
-		char fullname[MAX_READERNAME];
-		char *sSerialNumber;
-
 		sSerialNumber = libhal_device_get_property_string(ctx, udi,
 			"usb.serial", NULL);
-
-		(void)snprintf(fullname, sizeof(fullname), "%s (%s)",
-			driver->readerName, sSerialNumber);
-		readerTracker[i].fullName = strdup(fullname);
-	}
-	else
 #endif
-		readerTracker[i].fullName = strdup(driver->readerName);
+	
+	/* name from the Info.plist file */
+	strlcpy(fullname, driver->readerName, sizeof(fullname));
+
+	/* interface name from the device (if any) */
+	if (sInterfaceName)
+	{
+		strlcat(fullname, " [", sizeof(fullname));
+		strlcat(fullname, sInterfaceName, sizeof(fullname));
+		strlcat(fullname, "]", sizeof(fullname));
+		libhal_free_string(sInterfaceName);
+	}
+
+	/* serial number from the device (if any) */
+	if (sSerialNumber)
+	{
+		strlcat(fullname, " (", sizeof(fullname));
+		strlcat(fullname, sSerialNumber, sizeof(fullname));
+		strlcat(fullname, ")", sizeof(fullname));
+		libhal_free_string(sSerialNumber);
+	}
+
+	readerTracker[i].fullName = strdup(fullname);
 
 	ret = RFAddReader(readerTracker[i].fullName, PCSCLITE_HP_BASE_PORT + i,
 		driver->libraryPath, deviceName);
-	if (SCARD_S_SUCCESS != ret)
+	if ((SCARD_S_SUCCESS != ret) && (SCARD_E_UNKNOWN_READER != ret))
 	{
 		char *parent, *device_file;
 
@@ -455,7 +479,7 @@ error:
 		}
 	}
 
-	(void)SYS_MutexUnLock(&usbNotifierMutex);
+	(void)pthread_mutex_unlock(&usbNotifierMutex);
 } /* HPAddDevice */
 
 
@@ -479,7 +503,7 @@ static void HPRemoveDevice(/*@unused@*/ LibHalContext *ctx, const char *udi)
 	Log3(PCSC_LOG_INFO, "Removing USB device[%d]: %s", i,
 		short_name(readerTracker[i].udi));
 
-	(void)SYS_MutexLock(&usbNotifierMutex);
+	(void)pthread_mutex_lock(&usbNotifierMutex);
 
 	(void)RFRemoveReader(readerTracker[i].fullName, PCSCLITE_HP_BASE_PORT + i);
 	free(readerTracker[i].fullName);
@@ -487,7 +511,7 @@ static void HPRemoveDevice(/*@unused@*/ LibHalContext *ctx, const char *udi)
 	free(readerTracker[i].udi);
 	readerTracker[i].udi = NULL;
 
-	(void)SYS_MutexUnLock(&usbNotifierMutex);
+	(void)pthread_mutex_unlock(&usbNotifierMutex);
 
 	return;
 } /* HPRemoveDevice */
@@ -502,6 +526,8 @@ ULONG HPRegisterForHotplugEvents(void)
     int i, num_devices;
 	DBusError error;
 
+	(void)pthread_mutex_init(&usbNotifierMutex, NULL);
+
 	if (driverSize <= 0)
 	{
 		Log1(PCSC_LOG_INFO, "No bundle files in pcsc drivers directory: " PCSCLITE_HP_DROPDIR);
@@ -565,7 +591,7 @@ ULONG HPRegisterForHotplugEvents(void)
 
 	libhal_free_string_array(device_names);
 
-	(void)SYS_ThreadCreate(&usbNotifyThread, THREAD_ATTR_DETACHED,
+	(void)ThreadCreate(&usbNotifyThread, THREAD_ATTR_DETACHED,
 		(PCSCLITE_THREAD_FUNCTION( )) HPEstablishUSBNotifications, NULL);
 
 	return 0;
