@@ -3,10 +3,12 @@
  *
  * Copyright (C) 1999-2004
  *  David Corcoran <corcoran@linuxnet.com>
+ * Copyright (C) 2003-2004
  *  Damien Sauveron <damien.sauveron@labri.fr>
+ * Copyright (C) 2002-2009
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: ifdwrapper.c 3179 2008-10-24 07:26:59Z rousseau $
+ * $Id: ifdwrapper.c 4885 2010-04-18 12:17:56Z rousseau $
  */
 
 /**
@@ -15,6 +17,9 @@
  */
 
 #include <errno.h>
+#include <unistd.h>
+#include <pthread.h>
+
 #include "config.h"
 #include "misc.h"
 #include "pcscd.h"
@@ -27,13 +32,11 @@
 #include "sys_generic.h"
 #include "utils.h"
 
-#undef PCSCLITE_STATIC_DRIVER
-
 /**
  * Set the protocol type selection (PTS).
  * This function sets the appropriate protocol to be used on the card.
  */
-LONG IFDSetPTS(PREADER_CONTEXT rContext, DWORD dwProtocol, UCHAR ucFlags,
+LONG IFDSetPTS(READER_CONTEXT * rContext, DWORD dwProtocol, UCHAR ucFlags,
 	UCHAR ucPTS1, UCHAR ucPTS2, UCHAR ucPTS3)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
@@ -45,7 +48,7 @@ LONG IFDSetPTS(PREADER_CONTEXT rContext, DWORD dwProtocol, UCHAR ucFlags,
 	RESPONSECODE(*IFDH_set_protocol_parameters) (DWORD, DWORD, UCHAR,
 		UCHAR, UCHAR, UCHAR) = NULL;
 
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 	{
 		IFD_set_protocol_parameters = (RESPONSECODE(*)(DWORD, UCHAR, UCHAR,
 			UCHAR, UCHAR)) rContext->psFunctions.psFunctions_v1.pvfSetProtocolParameters;
@@ -67,40 +70,37 @@ LONG IFDSetPTS(PREADER_CONTEXT rContext, DWORD dwProtocol, UCHAR ucFlags,
 	/*
 	 * Locking is done in winscard.c SCardConnect() and SCardReconnect()
 	 *
-	 * This avoids to renegociate the protocol and confuse the card
+	 * This avoids to renegotiate the protocol and confuse the card
 	 * Error returned by CCID driver is: CCID_Receive Procedure byte conflict
 	 */
 
-	ucValue[0] = rContext->dwSlot;
+	ucValue[0] = rContext->slot;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 	{
-	        ucValue[0] = rContext->dwSlot;
-	        IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
-	        rv = (*IFD_set_protocol_parameters) (dwProtocol,
+		ucValue[0] = rContext->slot;
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		rv = (*IFD_set_protocol_parameters) (dwProtocol,
 			ucFlags, ucPTS1, ucPTS2, ucPTS3);
 	}
 	else
 	{
-		rv = (*IFDH_set_protocol_parameters) (rContext->dwSlot,
-						      dwProtocol,
-						      ucFlags, ucPTS1,
-						      ucPTS2, ucPTS3);
+		rv = (*IFDH_set_protocol_parameters) (rContext->slot,
+			dwProtocol, ucFlags, ucPTS1, ucPTS2, ucPTS3);
 	}
 #else
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+#ifdef IFDHANDLERv1
 	{
-	        ucValue[0] = rContext->dwSlot;
-	        IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		ucValue[0] = rContext->slot;
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = IFD_Set_Protocol_Parameters(dwProtocol, ucFlags, ucPTS1,
 			ucPTS2, ucPTS3);
 	}
-	else
-	{
-		rv = IFDHSetProtocolParameters(rContext->dwSlot, dwProtocol,
-			ucFlags, ucPTS1, ucPTS2, ucPTS3);
-	}
+#else
+	rv = IFDHSetProtocolParameters(rContext->slot, dwProtocol, ucFlags,
+		ucPTS1, ucPTS2, ucPTS3); 
+#endif
 #endif
 
 	return rv;
@@ -109,7 +109,7 @@ LONG IFDSetPTS(PREADER_CONTEXT rContext, DWORD dwProtocol, UCHAR ucFlags,
 /**
  * Open a communication channel to the IFD.
  */
-LONG IFDOpenIFD(PREADER_CONTEXT rContext)
+LONG IFDOpenIFD(READER_CONTEXT * rContext)
 {
 	RESPONSECODE rv = 0;
 
@@ -118,11 +118,11 @@ LONG IFDOpenIFD(PREADER_CONTEXT rContext)
 	RESPONSECODE(*IFDH_create_channel) (DWORD, DWORD) = NULL;
 	RESPONSECODE(*IFDH_create_channel_by_name) (DWORD, LPSTR) = NULL;
 
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 		IO_create_channel =
 			rContext->psFunctions.psFunctions_v1.pvfCreateChannel;
 	else
-		if (rContext->dwVersion == IFD_HVERSION_2_0)
+		if (rContext->version == IFD_HVERSION_2_0)
 			IFDH_create_channel =
 				rContext->psFunctions.psFunctions_v2.pvfCreateChannel;
 		else
@@ -135,42 +135,41 @@ LONG IFDOpenIFD(PREADER_CONTEXT rContext)
 #endif
 
 	/* LOCK THIS CODE REGION */
-	SYS_MutexLock(rContext->mMutex);
+	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 	{
-		rv = (*IO_create_channel) (rContext->dwPort);
-	} else if (rContext->dwVersion == IFD_HVERSION_2_0)
+		rv = (*IO_create_channel) (rContext->port);
+	} else if (rContext->version == IFD_HVERSION_2_0)
 	{
-		rv = (*IFDH_create_channel) (rContext->dwSlot, rContext->dwPort);
+		rv = (*IFDH_create_channel) (rContext->slot, rContext->port);
 	} else
 	{
 		/* use device name only if defined */
 		if (rContext->lpcDevice[0] != '\0')
-			rv = (*IFDH_create_channel_by_name) (rContext->dwSlot, rContext->lpcDevice);
+			rv = (*IFDH_create_channel_by_name) (rContext->slot, rContext->lpcDevice);
 		else
-			rv = (*IFDH_create_channel) (rContext->dwSlot, rContext->dwPort);
+			rv = (*IFDH_create_channel) (rContext->slot, rContext->port);
 	}
 #else
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
-	{
-		rv = IO_Create_Channel(rContext->dwPort);
-	} else if (rContext->dwVersion == IFD_HVERSION_2_0)
-	{
-		rv = IFDHCreateChannel(rContext->dwSlot, rContext->dwPort);
-	} else
+#ifdef IFDHANDLERv1
+	rv = IO_Create_Channel(rContext->port);
+#elif IFDHANDLERv2
+	rv = IFDHCreateChannel(rContext->slot, rContext->port);
+#else
 	{
 		/* Use device name only if defined */
 		if (rContext->lpcDevice[0] != '\0')
-			rv = IFDHCreateChannelByName(rContext->dwSlot, rContext->lpcDevice);
+			rv = IFDHCreateChannelByName(rContext->slot, rContext->lpcDevice);
 		else
-			rv = IFDHCreateChannel(rContext->dwSlot, rContext->dwPort);
+			rv = IFDHCreateChannel(rContext->slot, rContext->port);
 	}
+#endif
 #endif
 
 	/* END OF LOCKED REGION */
-	SYS_MutexUnLock(rContext->mMutex);
+	(void)pthread_mutex_unlock(rContext->mMutex);
 
 	return rv;
 }
@@ -178,7 +177,7 @@ LONG IFDOpenIFD(PREADER_CONTEXT rContext)
 /**
  * Close a communication channel to the IFD.
  */
-LONG IFDCloseIFD(PREADER_CONTEXT rContext)
+LONG IFDCloseIFD(READER_CONTEXT * rContext)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
 	int repeat;
@@ -187,7 +186,7 @@ LONG IFDCloseIFD(PREADER_CONTEXT rContext)
 	RESPONSECODE(*IO_close_channel) (void) = NULL;
 	RESPONSECODE(*IFDH_close_channel) (DWORD) = NULL;
 
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 		IO_close_channel = rContext->psFunctions.psFunctions_v1.pvfCloseChannel;
 	else
 		IFDH_close_channel = rContext->psFunctions.psFunctions_v2.pvfCloseChannel;
@@ -196,33 +195,34 @@ LONG IFDCloseIFD(PREADER_CONTEXT rContext)
 	/* TRY TO LOCK THIS CODE REGION */
 	repeat = 5;
 again:
-	rv = SYS_MutexTryLock(rContext->mMutex);
+	rv = pthread_mutex_trylock(rContext->mMutex);
 	if (EBUSY == rv)
 	{
 		Log1(PCSC_LOG_ERROR, "Locking failed");
 		repeat--;
 		if (repeat)
 		{
-			SYS_USleep(100*1000);	/* 100 ms */
+			(void)SYS_USleep(100*1000);	/* 100 ms */
 			goto again;
 		}
 	}
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 
 		rv = (*IO_close_channel) ();
 	else
-		rv = (*IFDH_close_channel) (rContext->dwSlot);
+		rv = (*IFDH_close_channel) (rContext->slot);
 #else
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
-		rv = IO_Close_Channel();
-	else
-		rv = IFDHCloseChannel(rContext->dwSlot);
+#ifdef IFDHANDLERv1
+	rv = IO_Close_Channel();
+#else
+	rv = IFDHCloseChannel(rContext->slot);
+#endif
 #endif
 
 	/* END OF LOCKED REGION */
-	SYS_MutexUnLock(rContext->mMutex);
+	(void)pthread_mutex_unlock(rContext->mMutex);
 
 	return rv;
 }
@@ -230,7 +230,7 @@ again:
 /**
  * Set capabilities in the reader.
  */
-LONG IFDSetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
+LONG IFDSetCapabilities(READER_CONTEXT * rContext, DWORD dwTag,
 			DWORD dwLength, PUCHAR pucValue)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
@@ -239,7 +239,7 @@ LONG IFDSetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
 	RESPONSECODE(*IFD_set_capabilities) (DWORD, PUCHAR) = NULL;
 	RESPONSECODE(*IFDH_set_capabilities) (DWORD, DWORD, DWORD, PUCHAR) = NULL;
 
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 		IFD_set_capabilities = rContext->psFunctions.psFunctions_v1.pvfSetCapabilities;
 	else
 		IFDH_set_capabilities = rContext->psFunctions.psFunctions_v2.pvfSetCapabilities;
@@ -251,17 +251,17 @@ LONG IFDSetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
 	 */
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 		rv = (*IFD_set_capabilities) (dwTag, pucValue);
 	else
-		rv = (*IFDH_set_capabilities) (rContext->dwSlot, dwTag,
+		rv = (*IFDH_set_capabilities) (rContext->slot, dwTag,
 			dwLength, pucValue);
 #else
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
-		rv = IFD_Set_Capabilities(dwTag, pucValue);
-	else
-		rv = IFDHSetCapabilities(rContext->dwSlot, dwTag, dwLength,
-			pucValue);
+#ifdef IFDHANDLERv1
+	rv = IFD_Set_Capabilities(dwTag, pucValue);
+#else
+	rv = IFDHSetCapabilities(rContext->slot, dwTag, dwLength, pucValue);
+#endif
 #endif
 
 	return rv;
@@ -272,16 +272,16 @@ LONG IFDSetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
  * Other functions int this file will call
  * the driver directly to not cause a deadlock.
  */
-LONG IFDGetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
+LONG IFDGetCapabilities(READER_CONTEXT * rContext, DWORD dwTag,
 	PDWORD pdwLength, PUCHAR pucValue)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFD_get_capabilities) (DWORD, PUCHAR) = NULL;
-	RESPONSECODE(*IFDH_get_capabilities) (DWORD, DWORD, PDWORD, PUCHAR) = NULL;
+	RESPONSECODE(*IFD_get_capabilities) (DWORD, /*@out@*/ PUCHAR) = NULL;
+	RESPONSECODE(*IFDH_get_capabilities) (DWORD, DWORD, PDWORD, /*@out@*/ PUCHAR) = NULL;
 
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 		IFD_get_capabilities =
 			rContext->psFunctions.psFunctions_v1.pvfGetCapabilities;
 	else
@@ -290,24 +290,24 @@ LONG IFDGetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
 #endif
 
 	/* LOCK THIS CODE REGION */
-	SYS_MutexLock(rContext->mMutex);
+	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 		rv = (*IFD_get_capabilities) (dwTag, pucValue);
 	else
-		rv = (*IFDH_get_capabilities) (rContext->dwSlot, dwTag,
+		rv = (*IFDH_get_capabilities) (rContext->slot, dwTag,
 			pdwLength, pucValue);
 #else
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
-		rv = IFD_Get_Capabilities(dwTag, pucValue);
-	else
-		rv = IFDHGetCapabilities(rContext->dwSlot, dwTag, pdwLength,
-			pucValue);
+#ifdef IFDHANDLERv1
+	rv = IFD_Get_Capabilities(dwTag, pucValue);
+#else
+	rv = IFDHGetCapabilities(rContext->slot, dwTag, pdwLength, pucValue);
+#endif
 #endif
 
 	/* END OF LOCKED REGION */
-	SYS_MutexUnLock(rContext->mMutex);
+	(void)pthread_mutex_unlock(rContext->mMutex);
 
 	return rv;
 }
@@ -315,12 +315,14 @@ LONG IFDGetCapabilities(PREADER_CONTEXT rContext, DWORD dwTag,
 /**
  * Power up/down or reset's an ICC located in the IFD.
  */
-LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
+LONG IFDPowerICC(READER_CONTEXT * rContext, DWORD dwAction,
 	PUCHAR pucAtr, PDWORD pdwAtrLen)
 {
 	RESPONSECODE rv;
+#ifndef PCSCLITE_STATIC_DRIVER
 	short ret;
 	SMARTCARD_EXTENSION sSmartCard;
+#endif
 	DWORD dwStatus;
 	UCHAR ucValue[1];
 
@@ -339,47 +341,48 @@ LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
 	/*
 	 * Check that the card is inserted first
 	 */
-	IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
+	(void)IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
 
 	if (dwStatus & SCARD_ABSENT)
 		return SCARD_W_REMOVED_CARD;
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 		IFD_power_icc = rContext->psFunctions.psFunctions_v1.pvfPowerICC;
 	else
 		IFDH_power_icc = rContext->psFunctions.psFunctions_v2.pvfPowerICC;
 #endif
 
 	/* LOCK THIS CODE REGION */
-	SYS_MutexLock(rContext->mMutex);
+	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 	{
-		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		ucValue[0] = rContext->slot;
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = (*IFD_power_icc) (dwAction);
 	}
 	else
 	{
-		rv = (*IFDH_power_icc) (rContext->dwSlot, dwAction,
+		rv = (*IFDH_power_icc) (rContext->slot, dwAction,
 			pucAtr, pdwAtrLen);
 
 		ret = ATRDecodeAtr(&sSmartCard, pucAtr, *pdwAtrLen);
 	}
 #else
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+#ifdef IFDHANDLERv1
 	{
-		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		ucValue[0] = rContext->slot;
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = IFD_Power_ICC(dwAction);
 	}
-	else
-		rv = IFDHPowerICC(rContext->dwSlot, dwAction, pucAtr, pdwAtrLen);
+#else
+	rv = IFDHPowerICC(rContext->slot, dwAction, pucAtr, pdwAtrLen);
+#endif
 #endif
 
 	/* END OF LOCKED REGION */
-	SYS_MutexUnLock(rContext->mMutex);
+	(void)pthread_mutex_unlock(rContext->mMutex);
 
 	/* use clean values in case of error */
 	if (rv != IFD_SUCCESS)
@@ -389,7 +392,7 @@ LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
 
 		if (rv == IFD_NO_SUCH_DEVICE)
 		{
-			SendHotplugSignal();
+			(void)SendHotplugSignal();
 			return SCARD_E_READER_UNAVAILABLE;
 		}
 
@@ -399,8 +402,8 @@ LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
 	/*
 	 * Get the ATR and it's length
 	 */
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
-		IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
+	if (rContext->version == IFD_HVERSION_1_0)
+		(void)IFDStatusICC(rContext, &dwStatus, pucAtr, pdwAtrLen);
 
 	return rv;
 }
@@ -409,7 +412,7 @@ LONG IFDPowerICC(PREADER_CONTEXT rContext, DWORD dwAction,
  * Provide statistical information about the IFD and ICC including insertions,
  * atr, powering status/etc.
  */
-LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
+LONG IFDStatusICC(READER_CONTEXT * rContext, PDWORD pdwStatus,
 	PUCHAR pucAtr, PDWORD pdwAtrLen)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
@@ -420,9 +423,9 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 #ifndef PCSCLITE_STATIC_DRIVER
 	RESPONSECODE(*IFD_is_icc_present) (void) = NULL;
 	RESPONSECODE(*IFDH_icc_presence) (DWORD) = NULL;
-	RESPONSECODE(*IFD_get_capabilities) (DWORD, PUCHAR) = NULL;
+	RESPONSECODE(*IFD_get_capabilities) (DWORD, /*@out@*/ PUCHAR) = NULL;
 
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 	{
 		IFD_is_icc_present =
 			rContext->psFunctions.psFunctions_v1.pvfICCPresence;
@@ -433,31 +436,32 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 		IFDH_icc_presence = rContext->psFunctions.psFunctions_v2.pvfICCPresence;
 #endif
 
-	/* LOCK THIS CODE REGION */ 
-	SYS_MutexLock(rContext->mMutex);
+	/* LOCK THIS CODE REGION */
+	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 	{
-		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		ucValue[0] = rContext->slot;
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = (*IFD_is_icc_present) ();
 	}
 	else
-		rv = (*IFDH_icc_presence) (rContext->dwSlot);
+		rv = (*IFDH_icc_presence) (rContext->slot);
 #else
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+#ifdef IFDHANDLERv1
 	{
-		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		ucValue[0] = rContext->slot;
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = IFD_Is_ICC_Present();
 	}
-	else
-		rv = IFDHICCPresence(rContext->dwSlot);
+#else
+	rv = IFDHICCPresence(rContext->slot);
+#endif
 #endif
 
 	/* END OF LOCKED REGION */
-	SYS_MutexUnLock(rContext->mMutex);
+	(void)pthread_mutex_unlock(rContext->mMutex);
 
 	if (rv == IFD_SUCCESS || rv == IFD_ICC_PRESENT)
 		dwCardStatus |= SCARD_PRESENT;
@@ -471,7 +475,7 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 
 			if (rv == IFD_NO_SUCH_DEVICE)
 			{
-				SendHotplugSignal();
+				(void)SendHotplugSignal();
 				return SCARD_E_READER_UNAVAILABLE;
 			}
 
@@ -484,7 +488,7 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 	 * to conserve resources
 	 */
 
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 	{
 		if (rv == IFD_SUCCESS || rv == IFD_ICC_PRESENT)
 		{
@@ -492,20 +496,22 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
 
 			dwTag = TAG_IFD_ATR;
 
-			/* LOCK THIS CODE REGION */ 
-			SYS_MutexLock(rContext->mMutex);
+			/* LOCK THIS CODE REGION */
+			(void)pthread_mutex_lock(rContext->mMutex);
 
-			ucValue[0] = rContext->dwSlot;
-			IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+			ucValue[0] = rContext->slot;
+			(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 
 #ifndef PCSCLITE_STATIC_DRIVER
 			rv = (*IFD_get_capabilities) (dwTag, pucAtr);
 #else
+#ifdef IFDHANDLERv1
 			rv = IFD_Get_Capabilities(dwTag, pucAtr);
+#endif
 #endif
 
 			/* END OF LOCKED REGION */
-			SYS_MutexUnLock(rContext->mMutex);
+			(void)pthread_mutex_unlock(rContext->mMutex);
 
 			/*
 			 * FIX :: This is a temporary way to return the correct size
@@ -549,16 +555,17 @@ LONG IFDStatusICC(PREADER_CONTEXT rContext, PDWORD pdwStatus,
  * Valid only for IFDHandler version 2.0
  */
 
-LONG IFDControl_v2(PREADER_CONTEXT rContext, PUCHAR TxBuffer,
+LONG IFDControl_v2(READER_CONTEXT * rContext, PUCHAR TxBuffer,
 	DWORD TxLength, PUCHAR RxBuffer, PDWORD RxLength)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	RESPONSECODE(*IFDH_control_v2) (DWORD, PUCHAR, DWORD, PUCHAR, PDWORD);
+	RESPONSECODE(*IFDH_control_v2) (DWORD, PUCHAR, DWORD, /*@out@*/ PUCHAR,
+		PDWORD);
 #endif
 
-	if (rContext->dwVersion != IFD_HVERSION_2_0)
+	if (rContext->version != IFD_HVERSION_2_0)
 		return SCARD_E_UNSUPPORTED_FEATURE;
 
 #ifndef PCSCLITE_STATIC_DRIVER
@@ -566,24 +573,26 @@ LONG IFDControl_v2(PREADER_CONTEXT rContext, PUCHAR TxBuffer,
 #endif
 
 	/* LOCK THIS CODE REGION */
-	SYS_MutexLock(rContext->mMutex);
+	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	rv = (*IFDH_control_v2) (rContext->dwSlot, TxBuffer, TxLength,
+	rv = (*IFDH_control_v2) (rContext->slot, TxBuffer, TxLength,
 		RxBuffer, RxLength);
-#else
-	rv = IFDHControl_v2(rContext->dwSlot, TxBuffer, TxLength,
+#elif IFDHANDLERv2
+	rv = IFDHControl(rContext->slot, TxBuffer, TxLength,
 		RxBuffer, RxLength);
 #endif
 
 	/* END OF LOCKED REGION */
-	SYS_MutexUnLock(rContext->mMutex);
+	(void)pthread_mutex_unlock(rContext->mMutex);
 
 	if (rv == IFD_SUCCESS)
 		return SCARD_S_SUCCESS;
 	else
 	{
 		Log2(PCSC_LOG_ERROR, "Card not transacted: %ld", rv);
+		LogXxd(PCSC_LOG_DEBUG, "TxBuffer ", TxBuffer, TxLength);
+		LogXxd(PCSC_LOG_DEBUG, "RxBuffer ", RxBuffer, *RxLength);
 		return SCARD_E_NOT_TRANSACTED;
 	}
 }
@@ -597,7 +606,7 @@ LONG IFDControl_v2(PREADER_CONTEXT rContext, PUCHAR TxBuffer,
  * Valid only for IFDHandler version 3.0 and up
  */
 
-LONG IFDControl(PREADER_CONTEXT rContext, DWORD ControlCode,
+LONG IFDControl(READER_CONTEXT * rContext, DWORD ControlCode,
 	LPCVOID TxBuffer, DWORD TxLength, LPVOID RxBuffer, DWORD RxLength,
 	LPDWORD BytesReturned)
 {
@@ -607,39 +616,49 @@ LONG IFDControl(PREADER_CONTEXT rContext, DWORD ControlCode,
 	RESPONSECODE(*IFDH_control) (DWORD, DWORD, LPCVOID, DWORD, LPVOID, DWORD, LPDWORD);
 #endif
 
-	if (rContext->dwVersion < IFD_HVERSION_3_0)
+	if (rContext->version < IFD_HVERSION_3_0)
 		return SCARD_E_UNSUPPORTED_FEATURE;
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	IFDH_control = rContext->psFunctions.psFunctions_v3.pvfControl;
 #endif
 
-	/* LOCK THIS CODE REGION */ 
-	SYS_MutexLock(rContext->mMutex);
+	/* LOCK THIS CODE REGION */
+	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	rv = (*IFDH_control) (rContext->dwSlot, ControlCode, TxBuffer,
+	rv = (*IFDH_control) (rContext->slot, ControlCode, TxBuffer,
 		TxLength, RxBuffer, RxLength, BytesReturned);
-#else
-	rv = IFDHControl(rContext->dwSlot, ControlCode, TxBuffer,
+#elif IFDHANDLERv3
+	rv = IFDHControl(rContext->slot, ControlCode, TxBuffer,
 		TxLength, RxBuffer, RxLength, BytesReturned);
 #endif
 
 	/* END OF LOCKED REGION */
-	SYS_MutexUnLock(rContext->mMutex);
+	(void)pthread_mutex_unlock(rContext->mMutex);
 
 	if (rv == IFD_SUCCESS)
 		return SCARD_S_SUCCESS;
 	else
 	{
 		Log2(PCSC_LOG_ERROR, "Card not transacted: %ld", rv);
+		Log3(PCSC_LOG_DEBUG, "ControlCode: 0x%.8LX BytesReturned: %ld",
+			ControlCode, *BytesReturned);
+		LogXxd(PCSC_LOG_DEBUG, "TxBuffer ", TxBuffer, TxLength);
+		LogXxd(PCSC_LOG_DEBUG, "RxBuffer ", RxBuffer, *BytesReturned);
 
 		if (rv == IFD_NO_SUCH_DEVICE)
 		{
-			SendHotplugSignal();
+			(void)SendHotplugSignal();
 			return SCARD_E_READER_UNAVAILABLE;
 		}
 
+		if ((IFD_ERROR_NOT_SUPPORTED == rv) || (IFD_NOT_SUPPORTED == rv))
+			return SCARD_E_UNSUPPORTED_FEATURE;
+
+        if (IFD_ERROR_INSUFFICIENT_BUFFER ==rv)
+            return SCARD_E_INSUFFICIENT_BUFFER;
+
 		return SCARD_E_NOT_TRANSACTED;
 	}
 }
@@ -647,25 +666,24 @@ LONG IFDControl(PREADER_CONTEXT rContext, DWORD ControlCode,
 /**
  * Transmit an APDU to the ICC.
  */
-LONG IFDTransmit(PREADER_CONTEXT rContext, SCARD_IO_HEADER pioTxPci,
+LONG IFDTransmit(READER_CONTEXT * rContext, SCARD_IO_HEADER pioTxPci,
 	PUCHAR pucTxBuffer, DWORD dwTxLength, PUCHAR pucRxBuffer,
 	PDWORD pdwRxLength, PSCARD_IO_HEADER pioRxPci)
 {
 	RESPONSECODE rv = IFD_SUCCESS;
-	UCHAR ucValue[1] = "\x00";
 
 #ifndef PCSCLITE_STATIC_DRIVER
 	RESPONSECODE(*IFD_transmit_to_icc) (SCARD_IO_HEADER, PUCHAR, DWORD,
-		PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
+		/*@out@*/ PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
 	RESPONSECODE(*IFDH_transmit_to_icc) (DWORD, SCARD_IO_HEADER, PUCHAR,
-		DWORD, PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
+		DWORD, /*@out@*/ PUCHAR, PDWORD, PSCARD_IO_HEADER) = NULL;
 #endif
 
 	/* log the APDU */
 	DebugLogCategory(DEBUG_CATEGORY_APDU, pucTxBuffer, dwTxLength);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 		IFD_transmit_to_icc =
 			rContext->psFunctions.psFunctions_v1.pvfTransmitToICC;
 	else
@@ -673,37 +691,42 @@ LONG IFDTransmit(PREADER_CONTEXT rContext, SCARD_IO_HEADER pioTxPci,
 			rContext->psFunctions.psFunctions_v2.pvfTransmitToICC;
 #endif
 
-	/* LOCK THIS CODE REGION */ 
-	SYS_MutexLock(rContext->mMutex);
+	/* LOCK THIS CODE REGION */
+	(void)pthread_mutex_lock(rContext->mMutex);
 
 #ifndef PCSCLITE_STATIC_DRIVER
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+	if (rContext->version == IFD_HVERSION_1_0)
 	{
-		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		UCHAR ucValue[1];
+
+		ucValue[0] = rContext->slot;
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = (*IFD_transmit_to_icc) (pioTxPci, (LPBYTE) pucTxBuffer,
 			dwTxLength, pucRxBuffer, pdwRxLength, pioRxPci);
 	}
 	else
-		rv = (*IFDH_transmit_to_icc) (rContext->dwSlot, pioTxPci,
+		rv = (*IFDH_transmit_to_icc) (rContext->slot, pioTxPci,
 			(LPBYTE) pucTxBuffer, dwTxLength,
 			pucRxBuffer, pdwRxLength, pioRxPci);
 #else
-	if (rContext->dwVersion == IFD_HVERSION_1_0)
+#ifdef IFDHANDLERv1
 	{
-		ucValue[0] = rContext->dwSlot;
-		IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
+		UCHAR ucValue[1];
+
+		ucValue[0] = rContext->slot;
+		(void)IFDSetCapabilities(rContext, TAG_IFD_SLOTNUM, 1, ucValue);
 		rv = IFD_Transmit_to_ICC(pioTxPci, (LPBYTE) pucTxBuffer,
 			dwTxLength, pucRxBuffer, pdwRxLength, pioRxPci);
 	}
-	else
-		rv = IFDHTransmitToICC(rContext->dwSlot, pioTxPci,
-			(LPBYTE) pucTxBuffer, dwTxLength,
-			pucRxBuffer, pdwRxLength, pioRxPci);
+#else
+	rv = IFDHTransmitToICC(rContext->slot, pioTxPci,
+		(LPBYTE) pucTxBuffer, dwTxLength,
+		pucRxBuffer, pdwRxLength, pioRxPci);
+#endif
 #endif
 
 	/* END OF LOCKED REGION */
-	SYS_MutexUnLock(rContext->mMutex);
+	(void)pthread_mutex_unlock(rContext->mMutex);
 
 	/* log the returned status word */
 	DebugLogCategory(DEBUG_CATEGORY_SW, pucRxBuffer, *pdwRxLength);
@@ -716,7 +739,7 @@ LONG IFDTransmit(PREADER_CONTEXT rContext, SCARD_IO_HEADER pioTxPci,
 
 		if (rv == IFD_NO_SUCH_DEVICE)
 		{
-			SendHotplugSignal();
+			(void)SendHotplugSignal();
 			return SCARD_E_READER_UNAVAILABLE;
 		}
 
