@@ -16,16 +16,11 @@ int pcsc_assert_wire_constants(void)
 
         /* Constants... */
 
-        LASSERTF(PROTOCOL_VERSION_MAJOR == 3," found %lld\n",
+        LASSERTF(PROTOCOL_VERSION_MAJOR == 4," found %lld\n",
                  (long long)PROTOCOL_VERSION_MAJOR);
         LASSERTF(PROTOCOL_VERSION_MINOR == 0," found %lld\n",
                  (long long)PROTOCOL_VERSION_MINOR);
 
-        LASSERTF(PCSCLITE_MSG_KEY_LEN == 16," found %lld\n",
-                 (long long)PCSCLITE_MSG_KEY_LEN);
-        LASSERTF(PCSCLITE_MAX_MESSAGE_SIZE == 2048," found %lld\n",
-                 (long long)PCSCLITE_MAX_MESSAGE_SIZE);
-
         LASSERTF(MAX_READERNAME == 100," found %lld\n",
                  (long long)MAX_READERNAME);
         LASSERTF(MAX_ATR_SIZE == 33," found %lld\n",
@@ -33,24 +28,6 @@ int pcsc_assert_wire_constants(void)
         LASSERTF(MAX_BUFFER_SIZE == 264," found %lld\n",
                  (long long)MAX_BUFFER_SIZE);
 
-        /* enum pcsc_adm_commands */
-        LASSERTF(CMD_FUNCTION == 241, " found %lld\n",
-                 (long long)CMD_FUNCTION);
-        LASSERTF(CMD_FAILED == 242, " found %lld\n",
-                 (long long)CMD_FAILED);
-        LASSERTF(CMD_SERVER_DIED == 243, " found %lld\n",
-                 (long long)CMD_SERVER_DIED);
-        LASSERTF(CMD_CLIENT_DIED == 244, " found %lld\n",
-                 (long long)CMD_CLIENT_DIED);
-        LASSERTF(CMD_READER_EVENT == 245, " found %lld\n",
-                 (long long)CMD_READER_EVENT);
-        LASSERTF(CMD_SYN == 246, " found %lld\n",
-                 (long long)CMD_SYN);
-        LASSERTF(CMD_ACK == 247, " found %lld\n",
-                 (long long)CMD_ACK);
-        LASSERTF(CMD_VERSION == 248, " found %lld\n",
-                 (long long)CMD_VERSION);
-
         /* enum pcsc_msg_commands */
         LASSERTF(SCARD_ESTABLISH_CONTEXT == 1, " found %lld\n",
                  (long long)SCARD_ESTABLISH_CONTEXT);
@@ -84,40 +61,16 @@ int pcsc_assert_wire_constants(void)
                  (long long)SCARD_GET_ATTRIB);
         LASSERTF(SCARD_SET_ATTRIB == 16, " found %lld\n",
                  (long long)SCARD_SET_ATTRIB);
+        LASSERTF(CMD_VERSION == 17, " found %lld\n",
+                 (long long)CMD_VERSION);
+        LASSERTF(CMD_GET_READERS_STATE == 18, " found %lld\n",
+                 (long long)CMD_GET_READERS_STATE);
+        LASSERTF(CMD_WAIT_READER_STATE_CHANGE == 19, " found %lld\n",
+                 (long long)CMD_WAIT_READER_STATE_CHANGE);
+        LASSERTF(CMD_STOP_WAITING_READER_STATE_CHANGE == 20, " found %lld\n",
+                 (long long)CMD_STOP_WAITING_READER_STATE_CHANGE);
         /* Types... */
 
-        /* Checks for struct rxSharedSegment */
-        LASSERTF((int)sizeof(struct rxSharedSegment) == 2088, " found %lld\n",
-                 (long long)(int)sizeof(struct rxSharedSegment));
-        LASSERTF((int)offsetof(struct rxSharedSegment, mtype) == 0, " found %lld\n",
-                 (long long)(int)offsetof(struct rxSharedSegment, mtype));
-        LASSERTF((int)sizeof(((struct rxSharedSegment *)0)->mtype) == 4, " found %lld\n",
-                 (long long)(int)sizeof(((struct rxSharedSegment *)0)->mtype));
-        LASSERTF((int)offsetof(struct rxSharedSegment, user_id) == 4, " found %lld\n",
-                 (long long)(int)offsetof(struct rxSharedSegment, user_id));
-        LASSERTF((int)sizeof(((struct rxSharedSegment *)0)->user_id) == 4, " found %lld\n",
-                 (long long)(int)sizeof(((struct rxSharedSegment *)0)->user_id));
-        LASSERTF((int)offsetof(struct rxSharedSegment, group_id) == 8, " found %lld\n",
-                 (long long)(int)offsetof(struct rxSharedSegment, group_id));
-        LASSERTF((int)sizeof(((struct rxSharedSegment *)0)->group_id) == 4, " found %lld\n",
-                 (long long)(int)sizeof(((struct rxSharedSegment *)0)->group_id));
-        LASSERTF((int)offsetof(struct rxSharedSegment, command) == 12, " found %lld\n",
-                 (long long)(int)offsetof(struct rxSharedSegment, command));
-        LASSERTF((int)sizeof(((struct rxSharedSegment *)0)->command) == 4, " found %lld\n",
-                 (long long)(int)sizeof(((struct rxSharedSegment *)0)->command));
-        LASSERTF((int)offsetof(struct rxSharedSegment, date) == 16, " found %lld\n",
-                 (long long)(int)offsetof(struct rxSharedSegment, date));
-        LASSERTF((int)sizeof(((struct rxSharedSegment *)0)->date) == 8, " found %lld\n",
-                 (long long)(int)sizeof(((struct rxSharedSegment *)0)->date));
-        LASSERTF((int)offsetof(struct rxSharedSegment, key) == 24, " found %lld\n",
-                 (long long)(int)offsetof(struct rxSharedSegment, key));
-        LASSERTF((int)sizeof(((struct rxSharedSegment *)0)->key) == 16, " found %lld\n",
-                 (long long)(int)sizeof(((struct rxSharedSegment *)0)->key));
-        LASSERTF((int)offsetof(struct rxSharedSegment, data) == 40, " found %lld\n",
-                 (long long)(int)offsetof(struct rxSharedSegment, data));
-        LASSERTF((int)sizeof(((struct rxSharedSegment *)0)->data) == 2048, " found %lld\n",
-                 (long long)(int)sizeof(((struct rxSharedSegment *)0)->data));
-
         /* Checks for struct version_struct */
         LASSERTF((int)sizeof(struct version_struct) == 12, " found %lld\n",
                  (long long)(int)sizeof(struct version_struct));
@@ -277,10 +230,10 @@ int pcsc_assert_wire_constants(void)
         /* Checks for struct cancel_struct */
         LASSERTF((int)sizeof(struct cancel_struct) == 8, " found %lld\n",
                  (long long)(int)sizeof(struct cancel_struct));
-        LASSERTF((int)offsetof(struct cancel_struct, hCard) == 0, " found %lld\n",
-                 (long long)(int)offsetof(struct cancel_struct, hCard));
-        LASSERTF((int)sizeof(((struct cancel_struct *)0)->hCard) == 4, " found %lld\n",
-                 (long long)(int)sizeof(((struct cancel_struct *)0)->hCard));
+        LASSERTF((int)offsetof(struct cancel_struct, hContext) == 0, " found %lld\n",
+                 (long long)(int)offsetof(struct cancel_struct, hContext));
+        LASSERTF((int)sizeof(((struct cancel_struct *)0)->hContext) == 4, " found %lld\n",
+                 (long long)(int)sizeof(((struct cancel_struct *)0)->hContext));
         LASSERTF((int)offsetof(struct cancel_struct, rv) == 4, " found %lld\n",
                  (long long)(int)offsetof(struct cancel_struct, rv));
         LASSERTF((int)sizeof(((struct cancel_struct *)0)->rv) == 4, " found %lld\n",
@@ -323,7 +276,7 @@ int pcsc_assert_wire_constants(void)
                  (long long)(int)sizeof(((struct status_struct *)0)->rv));
 
         /* Checks for struct transmit_struct */
-        LASSERTF((int)sizeof(struct transmit_struct) == 560, " found %lld\n",
+        LASSERTF((int)sizeof(struct transmit_struct) == 32, " found %lld\n",
                  (long long)(int)sizeof(struct transmit_struct));
         LASSERTF((int)offsetof(struct transmit_struct, hCard) == 0, " found %lld\n",
                  (long long)(int)offsetof(struct transmit_struct, hCard));
@@ -337,37 +290,29 @@ int pcsc_assert_wire_constants(void)
                  (long long)(int)offsetof(struct transmit_struct, ioSendPciLength));
         LASSERTF((int)sizeof(((struct transmit_struct *)0)->ioSendPciLength) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct transmit_struct *)0)->ioSendPciLength));
-        LASSERTF((int)offsetof(struct transmit_struct, pbSendBuffer) == 12, " found %lld\n",
-                 (long long)(int)offsetof(struct transmit_struct, pbSendBuffer));
-        LASSERTF((int)sizeof(((struct transmit_struct *)0)->pbSendBuffer) == 264, " found %lld\n",
-                 (long long)(int)sizeof(((struct transmit_struct *)0)->pbSendBuffer));
-        LASSERTF((int)offsetof(struct transmit_struct, cbSendLength) == 276, " found %lld\n",
+        LASSERTF((int)offsetof(struct transmit_struct, cbSendLength) == 12, " found %lld\n",
                  (long long)(int)offsetof(struct transmit_struct, cbSendLength));
         LASSERTF((int)sizeof(((struct transmit_struct *)0)->cbSendLength) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct transmit_struct *)0)->cbSendLength));
-        LASSERTF((int)offsetof(struct transmit_struct, ioRecvPciProtocol) == 280, " found %lld\n",
+        LASSERTF((int)offsetof(struct transmit_struct, ioRecvPciProtocol) == 16, " found %lld\n",
                  (long long)(int)offsetof(struct transmit_struct, ioRecvPciProtocol));
         LASSERTF((int)sizeof(((struct transmit_struct *)0)->ioRecvPciProtocol) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct transmit_struct *)0)->ioRecvPciProtocol));
-        LASSERTF((int)offsetof(struct transmit_struct, ioRecvPciLength) == 284, " found %lld\n",
+        LASSERTF((int)offsetof(struct transmit_struct, ioRecvPciLength) == 20, " found %lld\n",
                  (long long)(int)offsetof(struct transmit_struct, ioRecvPciLength));
         LASSERTF((int)sizeof(((struct transmit_struct *)0)->ioRecvPciLength) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct transmit_struct *)0)->ioRecvPciLength));
-        LASSERTF((int)offsetof(struct transmit_struct, pbRecvBuffer) == 288, " found %lld\n",
-                 (long long)(int)offsetof(struct transmit_struct, pbRecvBuffer));
-        LASSERTF((int)sizeof(((struct transmit_struct *)0)->pbRecvBuffer) == 264, " found %lld\n",
-                 (long long)(int)sizeof(((struct transmit_struct *)0)->pbRecvBuffer));
-        LASSERTF((int)offsetof(struct transmit_struct, pcbRecvLength) == 552, " found %lld\n",
+        LASSERTF((int)offsetof(struct transmit_struct, pcbRecvLength) == 24, " found %lld\n",
                  (long long)(int)offsetof(struct transmit_struct, pcbRecvLength));
         LASSERTF((int)sizeof(((struct transmit_struct *)0)->pcbRecvLength) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct transmit_struct *)0)->pcbRecvLength));
-        LASSERTF((int)offsetof(struct transmit_struct, rv) == 556, " found %lld\n",
+        LASSERTF((int)offsetof(struct transmit_struct, rv) == 28, " found %lld\n",
                  (long long)(int)offsetof(struct transmit_struct, rv));
         LASSERTF((int)sizeof(((struct transmit_struct *)0)->rv) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct transmit_struct *)0)->rv));
 
         /* Checks for struct control_struct */
-        LASSERTF((int)sizeof(struct control_struct) == 552, " found %lld\n",
+        LASSERTF((int)sizeof(struct control_struct) == 24, " found %lld\n",
                  (long long)(int)sizeof(struct control_struct));
         LASSERTF((int)offsetof(struct control_struct, hCard) == 0, " found %lld\n",
                  (long long)(int)offsetof(struct control_struct, hCard));
@@ -377,27 +322,19 @@ int pcsc_assert_wire_constants(void)
                  (long long)(int)offsetof(struct control_struct, dwControlCode));
         LASSERTF((int)sizeof(((struct control_struct *)0)->dwControlCode) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct control_struct *)0)->dwControlCode));
-        LASSERTF((int)offsetof(struct control_struct, pbSendBuffer) == 8, " found %lld\n",
-                 (long long)(int)offsetof(struct control_struct, pbSendBuffer));
-        LASSERTF((int)sizeof(((struct control_struct *)0)->pbSendBuffer) == 264, " found %lld\n",
-                 (long long)(int)sizeof(((struct control_struct *)0)->pbSendBuffer));
-        LASSERTF((int)offsetof(struct control_struct, cbSendLength) == 272, " found %lld\n",
+        LASSERTF((int)offsetof(struct control_struct, cbSendLength) == 8, " found %lld\n",
                  (long long)(int)offsetof(struct control_struct, cbSendLength));
         LASSERTF((int)sizeof(((struct control_struct *)0)->cbSendLength) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct control_struct *)0)->cbSendLength));
-        LASSERTF((int)offsetof(struct control_struct, pbRecvBuffer) == 276, " found %lld\n",
-                 (long long)(int)offsetof(struct control_struct, pbRecvBuffer));
-        LASSERTF((int)sizeof(((struct control_struct *)0)->pbRecvBuffer) == 264, " found %lld\n",
-                 (long long)(int)sizeof(((struct control_struct *)0)->pbRecvBuffer));
-        LASSERTF((int)offsetof(struct control_struct, cbRecvLength) == 540, " found %lld\n",
+        LASSERTF((int)offsetof(struct control_struct, cbRecvLength) == 12, " found %lld\n",
                  (long long)(int)offsetof(struct control_struct, cbRecvLength));
         LASSERTF((int)sizeof(((struct control_struct *)0)->cbRecvLength) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct control_struct *)0)->cbRecvLength));
-        LASSERTF((int)offsetof(struct control_struct, dwBytesReturned) == 544, " found %lld\n",
+        LASSERTF((int)offsetof(struct control_struct, dwBytesReturned) == 16, " found %lld\n",
                  (long long)(int)offsetof(struct control_struct, dwBytesReturned));
         LASSERTF((int)sizeof(((struct control_struct *)0)->dwBytesReturned) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct control_struct *)0)->dwBytesReturned));
-        LASSERTF((int)offsetof(struct control_struct, rv) == 548, " found %lld\n",
+        LASSERTF((int)offsetof(struct control_struct, rv) == 20, " found %lld\n",
                  (long long)(int)offsetof(struct control_struct, rv));
         LASSERTF((int)sizeof(((struct control_struct *)0)->rv) == 4, " found %lld\n",
                  (long long)(int)sizeof(((struct control_struct *)0)->rv));
