@@ -6,7 +6,7 @@
  * Copyright (C) 2004-2008
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: testpcsc.c 4233 2009-05-28 13:30:12Z rousseau $
+ * $Id: testpcsc.c 4703 2010-02-02 10:34:18Z rousseau $
  */
 
 /**
@@ -224,6 +224,9 @@ wait_for_card_again:
 		case SCARD_PROTOCOL_T1:
 			pioSendPci = *SCARD_PCI_T1;
 			break;
+		case SCARD_PROTOCOL_RAW:
+			pioSendPci = *SCARD_PCI_RAW;
+			break;
 		default:
 			printf("Unknown protocol\n");
 			return -1;
@@ -241,7 +244,7 @@ wait_for_card_again:
 	printf("Testing SCardTransmit\t\t: ");
 	rv = SCardTransmit(hCard, &pioSendPci, bSendBuffer, send_length,
 		&pioRecvPci, bRecvBuffer, &length);
-	printf("%s\n", pcsc_stringify_error(rv));
+	test_rv(rv, hContext, PANIC);
 	printf(" card response:" GREEN);
 	for (i=0; i<length; i++)
 		printf(" %02X", bRecvBuffer[i]);
@@ -272,6 +275,36 @@ wait_for_card_again:
 #endif
 	test_rv(rv, hContext, DONT_PANIC);
 
+	printf("Testing SCardGetAttrib\t\t: ");
+#ifdef USE_AUTOALLOCATE
+	pcbAttrLen = SCARD_AUTOALLOCATE;
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_DEVICE_FRIENDLY_NAME, (unsigned char *)&pbAttr,
+		&pcbAttrLen);
+#else
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_DEVICE_FRIENDLY_NAME, NULL, &pcbAttrLen);
+	test_rv(rv, hContext, DONT_PANIC);
+	if (rv == SCARD_S_SUCCESS)
+	{
+		printf("SCARD_ATTR_DEVICE_FRIENDLY_NAME length: " GREEN "%ld\n" NORMAL, pcbAttrLen);
+		pbAttr = malloc(pcbAttrLen);
+	}
+
+	printf("Testing SCardGetAttrib\t\t: ");
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_DEVICE_FRIENDLY_NAME, pbAttr, &pcbAttrLen);
+#endif
+	test_rv(rv, hContext, DONT_PANIC);
+	if (rv == SCARD_S_SUCCESS)
+		printf("SCARD_ATTR_DEVICE_FRIENDLY_NAME: " GREEN "%s\n" NORMAL, pbAttr);
+
+#ifdef USE_AUTOALLOCATE
+	printf("Testing SCardFreeMemory\t\t: ");
+	rv = SCardFreeMemory(hContext, pbAttr);
+	test_rv(rv, hContext, PANIC);
+#else
+	if (pbAttr)
+		free(pbAttr);
+#endif
+
 	printf("Testing SCardGetAttrib\t\t: ");
 #ifdef USE_AUTOALLOCATE
 	pcbAttrLen = SCARD_AUTOALLOCATE;
