@@ -1,7 +1,7 @@
 /*
  * MUSCLE SmartCard Development ( http://www.linuxnet.com )
  *
- * Copyright (C) 2006-2007
+ * Copyright (C) 2006-2009
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
  * $Id: pcscdaemon.c 2377 2007-02-05 13:13:56Z rousseau $
@@ -21,9 +21,10 @@
 #include <signal.h>
 #include <dirent.h>
 #include <fcntl.h>
+#include <pthread.h>
 
-#include "debug.h"
 #include "config.h"
+#include "debuglog.h"
 #include "utils.h"
 #include "pcscd.h"
 #include "sys_generic.h"
@@ -40,8 +41,8 @@ pid_t GetDaemonPid(void)
 	{
 		char pid_ascii[PID_ASCII_SIZE];
 
-		fgets(pid_ascii, PID_ASCII_SIZE, f);
-		fclose(f);
+		(void)fgets(pid_ascii, PID_ASCII_SIZE, f);
+		(void)fclose(f);
 
 		pid = atoi(pid_ascii);
 	}
@@ -70,87 +71,12 @@ int SendHotplugSignal(void)
 				pid, strerror(errno));
 			return EXIT_FAILURE ;
 		}
-		SYS_Sleep(1);
+		(void)SYS_Sleep(1);
 	}
 
 	return EXIT_SUCCESS;
 } /* SendHotplugSignal */
 
-/**
- * Sends an asynchronous event to any waiting client
- *
- * Just write 1 byte to any fifo in PCSCLITE_EVENTS_DIR and remove the file
- *
- * This function must be secured since the files are created by the library
- * or any non privileged process. We must not follow symlinks for example
- */
-int StatSynchronize(struct pubReaderStatesList *readerState)
-{
-	DIR *dir_fd;
-	struct dirent *dir;
-
-	if (readerState)
-		SYS_MMapSynchronize((void *)readerState, SYS_GetPageSize() );
-
-	dir_fd = opendir(PCSCLITE_EVENTS_DIR);
-	if (NULL == dir_fd)
-	{
-		Log2(PCSC_LOG_ERROR, "Can't opendir " PCSCLITE_EVENTS_DIR ": %s",
-			strerror(errno));
-		return -1;
-	}
-
-	while ((dir = readdir(dir_fd)) != NULL)
-	{
-		char filename[FILENAME_MAX];
-		int fd;
-		char buf[] = { '\0' };
-		struct stat fstat_buf;
-
-		if ('.' == dir->d_name[0])
-			continue;
-
-		snprintf(filename, sizeof(filename), "%s/%s", PCSCLITE_EVENTS_DIR,
-			dir->d_name);
-		Log2(PCSC_LOG_DEBUG, "status file: %s", filename);
-
-		fd = SYS_OpenFile(filename, O_WRONLY | O_APPEND | O_NONBLOCK, 0);
-		if (fd < 0)
-		{
-			/* ENXIO "No such device or address" is a normal error
-			 * if the client is no more listening the pipe */
-			Log3(ENXIO == errno ? PCSC_LOG_DEBUG : PCSC_LOG_ERROR,
-				"Can't open %s: %s", filename, strerror(errno));
-		}
-		else
-		{
-			if (fstat(fd, &fstat_buf))
-			{
-				Log3(PCSC_LOG_ERROR, "Can't fstat %s: %s", filename,
-					strerror(errno));
-			}
-			else
-			{
-				/* check that the file is a FIFO */
-				if (!(fstat_buf.st_mode & S_IFIFO))
-					Log2(PCSC_LOG_ERROR, "%s is not a fifo", filename);
-				else
-					SYS_WriteFile(fd, buf, sizeof(buf));
-			}
-
-			SYS_CloseFile(fd);
-		}
-
-		if (unlink(filename))
-			Log3(PCSC_LOG_ERROR, "Can't remove %s: %s", filename,
-			strerror(errno));
-	}
-	closedir(dir_fd);
-
-	return 0;
-} /* StatSynchronize */
-
-
 /**
  * Check is OpenCT is running and display a critical message if it is
  *
@@ -172,3 +98,46 @@ int CheckForOpenCT(void)
 	return 0;
 } /* CheckForOpenCT */
 
+/**
+ * return the difference (as long int) in µs between 2 struct timeval
+ * r = a - b
+ */
+long int time_sub(struct timeval *a, struct timeval *b)
+{
+	struct timeval r;
+	r.tv_sec = a -> tv_sec - b -> tv_sec;
+	r.tv_usec = a -> tv_usec - b -> tv_usec;
+	if (r.tv_usec < 0)
+	{
+		r.tv_sec--;
+		r.tv_usec += 1000000;
+	}
+
+	return r.tv_sec * 1000000 + r.tv_usec;
+} /* time_sub */
+
+int ThreadCreate(pthread_t * pthThread, int attributes,
+	PCSCLITE_THREAD_FUNCTION(pvFunction), LPVOID pvArg)
+{
+	pthread_attr_t attr;
+	int ret;
+
+	ret = pthread_attr_init(&attr);
+	if (ret)
+		return ret;
+
+	ret = pthread_attr_setdetachstate(&attr,
+		attributes & THREAD_ATTR_DETACHED ? PTHREAD_CREATE_DETACHED : PTHREAD_CREATE_JOINABLE);
+	if (ret)
+	{
+		(void)pthread_attr_destroy(&attr);
+		return ret;
+	}
+
+	ret = pthread_create(pthThread, &attr, pvFunction, pvArg);
+	if (ret)
+		return ret;
+
+	ret = pthread_attr_destroy(&attr);
+	return ret;
+}
