@@ -1,7 +1,7 @@
 /*
  * MUSCLE SmartCard Development ( http://www.linuxnet.com )
  *
- * Copyright (C) 2006-2007
+ * Copyright (C) 2006-2009
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
  * $Id: pcscdaemon.c 2377 2007-02-05 13:13:56Z rousseau $
@@ -21,10 +21,18 @@ int SendHotplugSignal(void);
 /* defined in winscard_clnt.c */
 LONG SCardCheckDaemonAvailability(void);
 
-int StatSynchronize(/*@null@*/ struct pubReaderStatesList *readerState);
-int StatSynchronizeContext(SCARDCONTEXT hContext);
-
 int CheckForOpenCT(void);
 
+long int time_sub(struct timeval *a, struct timeval *b);
+
+/* thread attributes */
+#define THREAD_ATTR_DEFAULT			0
+#define THREAD_ATTR_DETACHED		1
+
+#define PCSCLITE_THREAD_FUNCTION(f)      void *(*f)(void *)
+
+int ThreadCreate(pthread_t *, int, PCSCLITE_THREAD_FUNCTION( ),
+	/*@null@*/ LPVOID);
+
 #endif
 
