@@ -3,9 +3,10 @@
  *
  * Copyright (C) 1999-2004
  *  David Corcoran <corcoran@linuxnet.com>
+ * Copyright (C) 2002-2009
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard.c 3178 2008-10-23 15:34:16Z rousseau $
+ * $Id: winscard.c 4964 2010-05-27 10:12:34Z rousseau $
  */
 
 /**
@@ -82,6 +83,7 @@
 #include <stdlib.h>
 #include <sys/time.h>
 #include <string.h>
+#include <pthread.h>
 
 #include "pcscd.h"
 #include "winscard.h"
@@ -94,6 +96,8 @@
 #include "sys_generic.h"
 #include "eventhandler.h"
 #include "utils.h"
+#include "reader.h"
+#include "strlcpycat.h"
 
 #undef DO_PROFILE
 #ifdef DO_PROFILE
@@ -142,21 +146,6 @@ static void profile_start(const char *f)
 	gettimeofday(&profile_time_start, NULL);
 } /* profile_start */
 
-/* r = a - b */
-static long int time_sub(struct timeval *a, struct timeval *b)
-{
-	struct timeval r;
-	r.tv_sec = a -> tv_sec - b -> tv_sec;
-	r.tv_usec = a -> tv_usec - b -> tv_usec;
-	if (r.tv_usec < 0)
-	{
-		r.tv_sec--;
-		r.tv_usec += 1000000;
-	}
-
-	return r.tv_sec * 1000000 + r.tv_usec;
-} /* time_sub */
-
 
 static void profile_end(const char *f, int line)
 {
@@ -181,19 +170,6 @@ static void profile_end(const char *f, int line)
 /** used for backward compatibility */
 #define SCARD_PROTOCOL_ANY_OLD	 0x1000
 
-/** Some defines for context stack. */
-#define SCARD_LAST_CONTEXT       1
-/** Some defines for context stack. */
-#define SCARD_NO_CONTEXT         0
-/** Some defines for context stack. */
-#define SCARD_EXCLUSIVE_CONTEXT -1
-/** Some defines for context stack. */
-#define SCARD_NO_LOCK            0
-
-SCARD_IO_REQUEST g_rgSCardT0Pci = { SCARD_PROTOCOL_T0, sizeof(SCARD_IO_REQUEST) };
-SCARD_IO_REQUEST g_rgSCardT1Pci = { SCARD_PROTOCOL_T1, sizeof(SCARD_IO_REQUEST) };
-SCARD_IO_REQUEST g_rgSCardRawPci = { SCARD_PROTOCOL_RAW, sizeof(SCARD_IO_REQUEST) };
-
 /**
  * @brief Creates an Application Context for a client.
  *
@@ -214,9 +190,11 @@ SCARD_IO_REQUEST g_rgSCardRawPci = { SCARD_PROTOCOL_RAW, sizeof(SCARD_IO_REQUEST
  * @retval SCARD_E_INVALID_VALUE Invalid scope type passed (\ref SCARD_E_INVALID_VALUE)
  * @retval SCARD_E_INVALID_PARAMETER phContext is null (\ref SCARD_E_INVALID_PARAMETER)
  */
-LONG SCardEstablishContext(DWORD dwScope, LPCVOID pvReserved1,
-	LPCVOID pvReserved2, LPSCARDCONTEXT phContext)
+LONG SCardEstablishContext(DWORD dwScope, /*@unused@*/ LPCVOID pvReserved1,
+	/*@unused@*/ LPCVOID pvReserved2, LPSCARDCONTEXT phContext)
 {
+	(void)pvReserved1;
+	(void)pvReserved2;
 	/*
 	 * Check for NULL pointer
 	 */
@@ -237,7 +215,7 @@ LONG SCardEstablishContext(DWORD dwScope, LPCVOID pvReserved1,
 
 	*phContext = (PCSCLITE_SVC_IDENTITY + SYS_RandomInt(1, 65535));
 
-	Log2(PCSC_LOG_DEBUG, "Establishing Context: %d", *phContext);
+	Log2(PCSC_LOG_DEBUG, "Establishing Context: 0x%X", *phContext);
 
 	return SCARD_S_SUCCESS;
 }
@@ -248,29 +226,33 @@ LONG SCardReleaseContext(SCARDCONTEXT hContext)
 	 * Nothing to do here RPC layer will handle this
 	 */
 
-	Log2(PCSC_LOG_DEBUG, "Releasing Context: %d", hContext);
+	Log2(PCSC_LOG_DEBUG, "Releasing Context: 0x%X", hContext);
 
 	return SCARD_S_SUCCESS;
 }
 
-LONG SCardSetTimeout(SCARDCONTEXT hContext, DWORD dwTimeout)
+LONG SCardSetTimeout(/*@unused@*/ SCARDCONTEXT hContext,
+	/*@unused@*/ DWORD dwTimeout)
 {
 	/*
 	 * This is only used at the client side of an RPC call but just in
 	 * case someone calls it here
 	 */
 
+	(void)hContext;
+	(void)dwTimeout;
 	return SCARD_E_UNSUPPORTED_FEATURE;
 }
 
-LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
+LONG SCardConnect(/*@unused@*/ SCARDCONTEXT hContext, LPCSTR szReader,
 	DWORD dwShareMode, DWORD dwPreferredProtocols, LPSCARDHANDLE phCard,
 	LPDWORD pdwActiveProtocol)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 	DWORD dwStatus;
 
+	(void)hContext;
 	PROFILE_START
 
 	/*
@@ -320,7 +302,7 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 	/*
 	 * Connect if not exclusive mode
 	 */
-	if (rContext->dwContexts == SCARD_EXCLUSIVE_CONTEXT)
+	if (rContext->contexts == SCARD_EXCLUSIVE_CONTEXT)
 	{
 		Log1(PCSC_LOG_ERROR, "Error Reader Exclusive");
 		return SCARD_E_SHARING_VIOLATION;
@@ -329,11 +311,11 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 	/*
 	 * wait until a possible transaction is finished
 	 */
-	if (rContext->dwLockId != 0)
+	if (rContext->hLockId != 0)
 	{
 		Log1(PCSC_LOG_INFO, "Waiting for release of lock");
-		while (rContext->dwLockId != 0)
-			SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
+		while (rContext->hLockId != 0)
+			(void)SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
 		Log1(PCSC_LOG_INFO, "Lock released");
 	}
 
@@ -379,7 +361,7 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 		{
 			/* lock here instead in IFDSetPTS() to lock up to
 			 * setting rContext->readerState->cardProtocol */
-			SYS_MutexLock(rContext->mMutex);
+			(void)pthread_mutex_lock(rContext->mMutex);
 
 			/* the protocol is not yet set (no PPS yet) */
 			if (SCARD_PROTOCOL_UNDEFINED == rContext->readerState->cardProtocol)
@@ -405,24 +387,24 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 				/* keep cardProtocol = SCARD_PROTOCOL_UNDEFINED in case of error  */
 				if (SET_PROTOCOL_PPS_FAILED == ret)
 				{
-					SYS_MutexUnLock(rContext->mMutex);
+					(void)pthread_mutex_unlock(rContext->mMutex);
 					return SCARD_W_UNRESPONSIVE_CARD;
 				}
 
 				if (SET_PROTOCOL_WRONG_ARGUMENT == ret)
 				{
-					SYS_MutexUnLock(rContext->mMutex);
+					(void)pthread_mutex_unlock(rContext->mMutex);
 					return SCARD_E_PROTO_MISMATCH;
 				}
 
-				/* use negociated protocol */
+				/* use negotiated protocol */
 				rContext->readerState->cardProtocol = ret;
 
-				SYS_MutexUnLock(rContext->mMutex);
+				(void)pthread_mutex_unlock(rContext->mMutex);
 			}
 			else
 			{
-				SYS_MutexUnLock(rContext->mMutex);
+				(void)pthread_mutex_unlock(rContext->mMutex);
 
 				if (! (dwPreferredProtocols & rContext->readerState->cardProtocol))
 					return SCARD_E_PROTO_MISMATCH;
@@ -434,13 +416,22 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 
 	if (dwShareMode != SCARD_SHARE_DIRECT)
 	{
-		if ((*pdwActiveProtocol != SCARD_PROTOCOL_T0)
-			&& (*pdwActiveProtocol != SCARD_PROTOCOL_T1))
-			Log2(PCSC_LOG_ERROR, "Active Protocol: unknown %d",
-				*pdwActiveProtocol);
-		else
-			Log2(PCSC_LOG_DEBUG, "Active Protocol: T=%d",
-				(*pdwActiveProtocol == SCARD_PROTOCOL_T0) ? 0 : 1);
+		switch (*pdwActiveProtocol)
+		{
+			case SCARD_PROTOCOL_T0:
+			case SCARD_PROTOCOL_T1:
+				Log2(PCSC_LOG_DEBUG, "Active Protocol: T=%d",
+					(*pdwActiveProtocol == SCARD_PROTOCOL_T0) ? 0 : 1);
+				break;
+
+			case SCARD_PROTOCOL_RAW:
+				Log1(PCSC_LOG_DEBUG, "Active Protocol: RAW");
+				break;
+
+			default:
+				Log2(PCSC_LOG_ERROR, "Active Protocol: unknown %d",
+					*pdwActiveProtocol);
+		}
 	}
 	else
 		Log1(PCSC_LOG_DEBUG, "Direct access: no protocol selected");
@@ -461,14 +452,14 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 
 	if (dwShareMode == SCARD_SHARE_EXCLUSIVE)
 	{
-		if (rContext->dwContexts == SCARD_NO_CONTEXT)
+		if (rContext->contexts == SCARD_NO_CONTEXT)
 		{
-			rContext->dwContexts = SCARD_EXCLUSIVE_CONTEXT;
-			RFLockSharing(*phCard);
+			rContext->contexts = SCARD_EXCLUSIVE_CONTEXT;
+			(void)RFLockSharing(*phCard);
 		}
 		else
 		{
-			RFDestroyReaderHandle(*phCard);
+			(void)RFDestroyReaderHandle(*phCard);
 			*phCard = 0;
 			return SCARD_E_SHARING_VIOLATION;
 		}
@@ -478,7 +469,7 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 		/*
 		 * Add a connection to the context stack
 		 */
-		rContext->dwContexts += 1;
+		rContext->contexts += 1;
 	}
 
 	/*
@@ -491,12 +482,12 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 		/*
 		 * Clean up - there is no more room
 		 */
-		RFDestroyReaderHandle(*phCard);
-		if (rContext->dwContexts == SCARD_EXCLUSIVE_CONTEXT)
-			rContext->dwContexts = SCARD_NO_CONTEXT;
+		(void)RFDestroyReaderHandle(*phCard);
+		if (rContext->contexts == SCARD_EXCLUSIVE_CONTEXT)
+			rContext->contexts = SCARD_NO_CONTEXT;
 		else
-			if (rContext->dwContexts > SCARD_NO_CONTEXT)
-				rContext->dwContexts -= 1;
+			if (rContext->contexts > SCARD_NO_CONTEXT)
+				rContext->contexts -= 1;
 
 		*phCard = 0;
 
@@ -506,10 +497,9 @@ LONG SCardConnect(SCARDCONTEXT hContext, LPCSTR szReader,
 	}
 
 	/*
-	 * Propagate new state to Shared Memory
+	 * Propagate new state to reader state
 	 */
-	rContext->readerState->readerSharing = rContext->dwContexts;
-	StatSynchronize(rContext->readerState);
+	rContext->readerState->readerSharing = rContext->contexts;
 
 	PROFILE_END
 
@@ -521,7 +511,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 	LPDWORD pdwActiveProtocol)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 
 	Log1(PCSC_LOG_DEBUG, "Attempting reconnect to token.");
 
@@ -573,14 +563,16 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	/*
-	 * RFUnblockReader( rContext ); FIX - this doesn't work
-	 */
-
 	if (dwInitialization == SCARD_RESET_CARD ||
 		dwInitialization == SCARD_UNPOWER_CARD)
 	{
 		DWORD dwAtrLen;
+
+		/*
+		 * Notify the card has been reset
+		 */
+		(void)RFSetReaderEventState(rContext, SCARD_RESET);
+
 		/*
 		 * Currently pcsc-lite keeps the card powered constantly
 		 */
@@ -604,87 +596,48 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 		rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 
 		/*
-		 * Notify the card has been reset
-		 * Not doing this could result in deadlock
+		 * Set up the status bit masks on dwStatus
 		 */
-		rv = RFCheckReaderEventState(rContext, hCard);
-		switch(rv)
+		if (rv == SCARD_S_SUCCESS)
 		{
-			/* avoid deadlock */
-			case SCARD_W_RESET_CARD:
-				break;
+			rContext->readerState->readerState |= SCARD_PRESENT;
+			rContext->readerState->readerState &= ~SCARD_ABSENT;
+			rContext->readerState->readerState |= SCARD_POWERED;
+			rContext->readerState->readerState |= SCARD_NEGOTIABLE;
+			rContext->readerState->readerState &= ~SCARD_SPECIFIC;
+			rContext->readerState->readerState &= ~SCARD_SWALLOWED;
+			rContext->readerState->readerState &= ~SCARD_UNKNOWN;
+		}
+		else
+		{
+			rContext->readerState->readerState |= SCARD_PRESENT;
+			rContext->readerState->readerState &= ~SCARD_ABSENT;
+			rContext->readerState->readerState |= SCARD_SWALLOWED;
+			rContext->readerState->readerState &= ~SCARD_POWERED;
+			rContext->readerState->readerState &= ~SCARD_NEGOTIABLE;
+			rContext->readerState->readerState &= ~SCARD_SPECIFIC;
+			rContext->readerState->readerState &= ~SCARD_UNKNOWN;
+			rContext->readerState->cardAtrLength = 0;
+		}
 
-			case SCARD_W_REMOVED_CARD:
-				Log1(PCSC_LOG_ERROR, "card removed");
-				return SCARD_W_REMOVED_CARD;
+		if (rContext->readerState->cardAtrLength > 0)
+		{
+			Log1(PCSC_LOG_DEBUG, "Reset complete.");
+			LogXxd(PCSC_LOG_DEBUG, "Card ATR: ",
+				rContext->readerState->cardAtr,
+				rContext->readerState->cardAtrLength);
+		}
+		else
+		{
+			DWORD dwStatus, dwAtrLen2;
+			UCHAR ucAtr[MAX_ATR_SIZE];
 
-			/* invalid EventStatus */
-			case SCARD_E_INVALID_VALUE:
-				Log1(PCSC_LOG_ERROR, "invalid EventStatus");
-				return SCARD_F_INTERNAL_ERROR;
-
-			/* invalid hCard, but hCard was widely used some lines above :( */
-			case SCARD_E_INVALID_HANDLE:
-				Log1(PCSC_LOG_ERROR, "invalid handle");
-				return SCARD_F_INTERNAL_ERROR;
-
-			case SCARD_S_SUCCESS:
-				/*
-				 * Notify the card has been reset
-				 */
-				RFSetReaderEventState(rContext, SCARD_RESET);
-
-				/*
-				 * Set up the status bit masks on dwStatus
-				 */
-				if (rv == SCARD_S_SUCCESS)
-				{
-					rContext->readerState->readerState |= SCARD_PRESENT;
-					rContext->readerState->readerState &= ~SCARD_ABSENT;
-					rContext->readerState->readerState |= SCARD_POWERED;
-					rContext->readerState->readerState |= SCARD_NEGOTIABLE;
-					rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-					rContext->readerState->readerState &= ~SCARD_SWALLOWED;
-					rContext->readerState->readerState &= ~SCARD_UNKNOWN;
-				}
-				else
-				{
-					rContext->readerState->readerState |= SCARD_PRESENT;
-					rContext->readerState->readerState &= ~SCARD_ABSENT;
-					rContext->readerState->readerState |= SCARD_SWALLOWED;
-					rContext->readerState->readerState &= ~SCARD_POWERED;
-					rContext->readerState->readerState &= ~SCARD_NEGOTIABLE;
-					rContext->readerState->readerState &= ~SCARD_SPECIFIC;
-					rContext->readerState->readerState &= ~SCARD_UNKNOWN;
-					rContext->readerState->cardAtrLength = 0;
-				}
-
-				if (rContext->readerState->cardAtrLength > 0)
-				{
-					Log1(PCSC_LOG_DEBUG, "Reset complete.");
-					LogXxd(PCSC_LOG_DEBUG, "Card ATR: ",
-						rContext->readerState->cardAtr,
-						rContext->readerState->cardAtrLength);
-				}
-				else
-				{
-					DWORD dwStatus, dwAtrLen2;
-					UCHAR ucAtr[MAX_ATR_SIZE];
-
-					Log1(PCSC_LOG_ERROR, "Error resetting card.");
-					IFDStatusICC(rContext, &dwStatus, ucAtr, &dwAtrLen2);
-					if (dwStatus & SCARD_PRESENT)
-						return SCARD_W_UNRESPONSIVE_CARD;
-					else
-						return SCARD_E_NO_SMARTCARD;
-				}
-				break;
-
-			default:
-				Log2(PCSC_LOG_ERROR,
-					"invalid retcode from RFCheckReaderEventState (%X)", rv);
-				return SCARD_F_INTERNAL_ERROR;
-				break;
+			Log1(PCSC_LOG_ERROR, "Error resetting card.");
+			(void)IFDStatusICC(rContext, &dwStatus, ucAtr, &dwAtrLen2);
+			if (dwStatus & SCARD_PRESENT)
+				return SCARD_W_UNRESPONSIVE_CARD;
+			else
+				return SCARD_E_NO_SMARTCARD;
 		}
 	}
 	else
@@ -709,7 +662,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 		{
 			/* lock here instead in IFDSetPTS() to lock up to
 			 * setting rContext->readerState->cardProtocol */
-			SYS_MutexLock(rContext->mMutex);
+			(void)pthread_mutex_lock(rContext->mMutex);
 
 			/* the protocol is not yet set (no PPS yet) */
 			if (SCARD_PROTOCOL_UNDEFINED == rContext->readerState->cardProtocol)
@@ -733,24 +686,24 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 				/* keep cardProtocol = SCARD_PROTOCOL_UNDEFINED in case of error  */
 				if (SET_PROTOCOL_PPS_FAILED == ret)
 				{
-					SYS_MutexUnLock(rContext->mMutex);
+					(void)pthread_mutex_unlock(rContext->mMutex);
 					return SCARD_W_UNRESPONSIVE_CARD;
 				}
 
 				if (SET_PROTOCOL_WRONG_ARGUMENT == ret)
 				{
-					SYS_MutexUnLock(rContext->mMutex);
+					(void)pthread_mutex_unlock(rContext->mMutex);
 					return SCARD_E_PROTO_MISMATCH;
 				}
 
-				/* use negociated protocol */
+				/* use negotiated protocol */
 				rContext->readerState->cardProtocol = ret;
 
-				SYS_MutexUnLock(rContext->mMutex);
+				(void)pthread_mutex_unlock(rContext->mMutex);
 			}
 			else
 			{
-				SYS_MutexUnLock(rContext->mMutex);
+				(void)pthread_mutex_unlock(rContext->mMutex);
 
 				if (! (dwPreferredProtocols & rContext->readerState->cardProtocol))
 					return SCARD_E_PROTO_MISMATCH;
@@ -760,19 +713,41 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 
 	*pdwActiveProtocol = rContext->readerState->cardProtocol;
 
+	if (dwShareMode != SCARD_SHARE_DIRECT)
+	{
+		switch (*pdwActiveProtocol)
+		{
+			case SCARD_PROTOCOL_T0:
+			case SCARD_PROTOCOL_T1:
+				Log2(PCSC_LOG_DEBUG, "Active Protocol: T=%d",
+					(*pdwActiveProtocol == SCARD_PROTOCOL_T0) ? 0 : 1);
+				break;
+
+			case SCARD_PROTOCOL_RAW:
+				Log1(PCSC_LOG_DEBUG, "Active Protocol: RAW");
+				break;
+
+			default:
+				Log2(PCSC_LOG_ERROR, "Active Protocol: unknown %d",
+					*pdwActiveProtocol);
+		}
+	}
+	else
+		Log1(PCSC_LOG_DEBUG, "Direct access: no protocol selected");
+
 	if (dwShareMode == SCARD_SHARE_EXCLUSIVE)
 	{
-		if (rContext->dwContexts == SCARD_EXCLUSIVE_CONTEXT)
+		if (rContext->contexts == SCARD_EXCLUSIVE_CONTEXT)
 		{
 			/*
 			 * Do nothing - we are already exclusive
 			 */
 		} else
 		{
-			if (rContext->dwContexts == SCARD_LAST_CONTEXT)
+			if (rContext->contexts == SCARD_LAST_CONTEXT)
 			{
-				rContext->dwContexts = SCARD_EXCLUSIVE_CONTEXT;
-				RFLockSharing(hCard);
+				rContext->contexts = SCARD_EXCLUSIVE_CONTEXT;
+				(void)RFLockSharing(hCard);
 			} else
 			{
 				return SCARD_E_SHARING_VIOLATION;
@@ -780,7 +755,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 		}
 	} else if (dwShareMode == SCARD_SHARE_SHARED)
 	{
-		if (rContext->dwContexts != SCARD_EXCLUSIVE_CONTEXT)
+		if (rContext->contexts != SCARD_EXCLUSIVE_CONTEXT)
 		{
 			/*
 			 * Do nothing - in sharing mode already
@@ -790,12 +765,12 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			/*
 			 * We are in exclusive mode but want to share now
 			 */
-			RFUnlockSharing(hCard);
-			rContext->dwContexts = SCARD_LAST_CONTEXT;
+			(void)RFUnlockSharing(hCard);
+			rContext->contexts = SCARD_LAST_CONTEXT;
 		}
 	} else if (dwShareMode == SCARD_SHARE_DIRECT)
 	{
-		if (rContext->dwContexts != SCARD_EXCLUSIVE_CONTEXT)
+		if (rContext->contexts != SCARD_EXCLUSIVE_CONTEXT)
 		{
 			/*
 			 * Do nothing - in sharing mode already
@@ -805,8 +780,8 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 			/*
 			 * We are in exclusive mode but want to share now
 			 */
-			RFUnlockSharing(hCard);
-			rContext->dwContexts = SCARD_LAST_CONTEXT;
+			(void)RFUnlockSharing(hCard);
+			rContext->contexts = SCARD_LAST_CONTEXT;
 		}
 	} else
 		return SCARD_E_INVALID_VALUE;
@@ -814,13 +789,12 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 	/*
 	 * Clear a previous event to the application
 	 */
-	RFClearReaderEventState(rContext, hCard);
+	(void)RFClearReaderEventState(rContext, hCard);
 
 	/*
-	 * Propagate new state to Shared Memory
+	 * Propagate new state to reader state
 	 */
-	rContext->readerState->readerSharing = rContext->dwContexts;
-	StatSynchronize(rContext->readerState);
+	rContext->readerState->readerSharing = rContext->contexts;
 
 	return SCARD_S_SUCCESS;
 }
@@ -828,7 +802,7 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 	DWORD dwAtrLen;
 
 	if (hCard == 0)
@@ -851,12 +825,12 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * wait until a possible transaction is finished
 	 */
-	if ((dwDisposition != SCARD_LEAVE_CARD) && (rContext->dwLockId != 0)
-		&& (rContext->dwLockId != hCard))
+	if ((dwDisposition != SCARD_LEAVE_CARD) && (rContext->hLockId != 0)
+		&& (rContext->hLockId != hCard))
 	{
 		Log1(PCSC_LOG_INFO, "Waiting for release of lock");
-		while (rContext->dwLockId != 0)
-			SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
+		while (rContext->hLockId != 0)
+			(void)SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
 		Log1(PCSC_LOG_INFO, "Lock released");
 	}
 
@@ -867,15 +841,20 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * Unlock any blocks on this context
 	 */
-	rv = RFUnlockSharing(hCard);
+	rv = RFUnlockAllSharing(hCard);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	Log2(PCSC_LOG_DEBUG, "Active Contexts: %d", rContext->dwContexts);
+	Log2(PCSC_LOG_DEBUG, "Active Contexts: %d", rContext->contexts);
 
 	if (dwDisposition == SCARD_RESET_CARD ||
 		dwDisposition == SCARD_UNPOWER_CARD)
 	{
+		/*
+		 * Notify the card has been reset
+		 */
+		(void)RFSetReaderEventState(rContext, SCARD_RESET);
+
 		/*
 		 * Currently pcsc-lite keeps the card powered constantly
 		 */
@@ -898,11 +877,6 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 		/* the protocol is unset after a power on */
 		rContext->readerState->cardProtocol = SCARD_PROTOCOL_UNDEFINED;
 
-		/*
-		 * Notify the card has been reset
-		 */
-		RFSetReaderEventState(rContext, SCARD_RESET);
-
 		/*
 		 * Set up the status bit masks on dwStatus
 		 */
@@ -947,7 +921,7 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 		 */
 		controlBuffer[0] = 0x20;
 		controlBuffer[1] = 0x15;
-		controlBuffer[2] = (rContext->dwSlot & 0x0000FFFF) + 1;
+		controlBuffer[2] = (rContext->slot & 0x0000FFFF) + 1;
 		controlBuffer[3] = 0x00;
 		controlBuffer[4] = 0x00;
 		receiveLength = 2;
@@ -980,30 +954,29 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * Remove and destroy this handle
 	 */
-	RFRemoveReaderHandle(rContext, hCard);
-	RFDestroyReaderHandle(hCard);
+	(void)RFRemoveReaderHandle(rContext, hCard);
+	(void)RFDestroyReaderHandle(hCard);
 
 	/*
 	 * For exclusive connection reset it to no connections
 	 */
-	if (rContext->dwContexts == SCARD_EXCLUSIVE_CONTEXT)
-		rContext->dwContexts = SCARD_NO_CONTEXT;
+	if (rContext->contexts == SCARD_EXCLUSIVE_CONTEXT)
+		rContext->contexts = SCARD_NO_CONTEXT;
 	else
 	{
 		/*
 		 * Remove a connection from the context stack
 		 */
-		rContext->dwContexts -= 1;
+		rContext->contexts -= 1;
 
-		if (rContext->dwContexts < 0)
-			rContext->dwContexts = 0;
+		if (rContext->contexts < 0)
+			rContext->contexts = 0;
 	}
 
 	/*
-	 * Propagate new state to Shared Memory
+	 * Propagate new state to reader state
 	 */
-	rContext->readerState->readerSharing = rContext->dwContexts;
-	StatSynchronize(rContext->readerState);
+	rContext->readerState->readerSharing = rContext->contexts;
 
 	return SCARD_S_SUCCESS;
 }
@@ -1011,7 +984,7 @@ LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition)
 LONG SCardBeginTransaction(SCARDHANDLE hCard)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext;
+	READER_CONTEXT * rContext;
 
 	if (hCard == 0)
 		return SCARD_E_INVALID_HANDLE;
@@ -1046,7 +1019,7 @@ LONG SCardBeginTransaction(SCARDHANDLE hCard)
 	/* if the transaction is not yet ready we sleep a bit so the client
 	 * do not retry immediately */
 	if (SCARD_E_SHARING_VIOLATION == rv)
-		SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
+		(void)SYS_USleep(PCSCLITE_LOCK_POLL_RATE);
 
 	Log2(PCSC_LOG_DEBUG, "Status: 0x%08X", rv);
 
@@ -1056,7 +1029,7 @@ LONG SCardBeginTransaction(SCARDHANDLE hCard)
 LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 	DWORD dwAtrLen;
 
 	/*
@@ -1117,7 +1090,7 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 		/*
 		 * Notify the card has been reset
 		 */
-		RFSetReaderEventState(rContext, SCARD_RESET);
+		(void)RFSetReaderEventState(rContext, SCARD_RESET);
 
 		/*
 		 * Set up the status bit masks on dwStatus
@@ -1164,7 +1137,7 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 		 */
 		controlBuffer[0] = 0x20;
 		controlBuffer[1] = 0x15;
-		controlBuffer[2] = (rContext->dwSlot & 0x0000FFFF) + 1;
+		controlBuffer[2] = (rContext->slot & 0x0000FFFF) + 1;
 		controlBuffer[3] = 0x00;
 		controlBuffer[4] = 0x00;
 		receiveLength = 2;
@@ -1197,7 +1170,7 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 	/*
 	 * Unlock any blocks on this context
 	 */
-	RFUnlockSharing(hCard);
+	(void)RFUnlockSharing(hCard);
 
 	Log2(PCSC_LOG_DEBUG, "Status: 0x%08X", rv);
 
@@ -1207,7 +1180,7 @@ LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition)
 LONG SCardCancelTransaction(SCARDHANDLE hCard)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 
 	/*
 	 * Ignoring dwDisposition for now
@@ -1245,11 +1218,17 @@ LONG SCardStatus(SCARDHANDLE hCard, LPSTR mszReaderNames,
 	LPDWORD pdwProtocol, LPBYTE pbAtr, LPDWORD pcbAtrLen)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 
 	if (hCard == 0)
 		return SCARD_E_INVALID_HANDLE;
 
+	/*
+	 * Make sure no one has a lock on this reader
+	 */
+	if ((rv = RFCheckSharing(hCard)) != SCARD_S_SUCCESS)
+		return rv;
+
 	rv = RFReaderInfoById(hCard, &rContext);
 
 	/*
@@ -1355,12 +1334,18 @@ LONG SCardStatus(SCARDHANDLE hCard, LPSTR mszReaderNames,
 	return rv;
 }
 
-LONG SCardGetStatusChange(SCARDCONTEXT hContext, DWORD dwTimeout,
-	LPSCARD_READERSTATE_A rgReaderStates, DWORD cReaders)
+LONG SCardGetStatusChange(/*@unused@*/ SCARDCONTEXT hContext,
+	/*@unused@*/ DWORD dwTimeout,
+	/*@unused@*/ LPSCARD_READERSTATE_A rgReaderStates,
+	/*@unused@*/ DWORD cReaders)
 {
 	/*
 	 * Client side function
 	 */
+	(void)hContext;
+	(void)dwTimeout;
+	(void)rgReaderStates;
+	(void)cReaders;
 	return SCARD_S_SUCCESS;
 }
 
@@ -1369,7 +1354,7 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode,
 	LPVOID pbRecvBuffer, DWORD cbRecvLength, LPDWORD lpBytesReturned)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 
 	/* 0 bytes returned by default */
 	*lpBytesReturned = 0;
@@ -1387,7 +1372,7 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode,
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	if (IFD_HVERSION_2_0 == rContext->dwVersion)
+	if (IFD_HVERSION_2_0 == rContext->version)
 		if (NULL == pbSendBuffer || 0 == cbSendLength)
 			return SCARD_E_INVALID_PARAMETER;
 
@@ -1402,13 +1387,7 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode,
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	/*
-	 * Make sure some event has not occurred
-	 */
-	if ((rv = RFCheckReaderEventState(rContext, hCard)) != SCARD_S_SUCCESS)
-		return rv;
-
-	if (IFD_HVERSION_2_0 == rContext->dwVersion)
+	if (IFD_HVERSION_2_0 == rContext->version)
 	{
 		/* we must wrap a API 3.0 client in an API 2.0 driver */
 		*lpBytesReturned = cbRecvLength;
@@ -1416,7 +1395,7 @@ LONG SCardControl(SCARDHANDLE hCard, DWORD dwControlCode,
 			cbSendLength, pbRecvBuffer, lpBytesReturned);
 	}
 	else
-		if (IFD_HVERSION_3_0 == rContext->dwVersion)
+		if (IFD_HVERSION_3_0 == rContext->version)
 			return IFDControl(rContext, dwControlCode, pbSendBuffer,
 				cbSendLength, pbRecvBuffer, cbRecvLength, lpBytesReturned);
 		else
@@ -1427,7 +1406,7 @@ LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	LPBYTE pbAttr, LPDWORD pcbAttrLen)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 
 	if (0 == hCard)
 		return SCARD_E_INVALID_HANDLE;
@@ -1460,20 +1439,48 @@ LONG SCardGetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 		return rv;
 
 	rv = IFDGetCapabilities(rContext, dwAttrId, pcbAttrLen, pbAttr);
-	if (rv == IFD_SUCCESS)
-		return SCARD_S_SUCCESS;
-	else
-		if (rv == IFD_ERROR_TAG)
-			return SCARD_E_UNSUPPORTED_FEATURE;
-		else
-			return SCARD_E_NOT_TRANSACTED;
+	switch(rv)
+	{
+		case IFD_SUCCESS:
+			rv = SCARD_S_SUCCESS;
+			break;
+		case IFD_ERROR_TAG:
+			/* Special case SCARD_ATTR_DEVICE_FRIENDLY_NAME as it is better
+			 * implemented in pcscd (it knows the friendly name)
+			 */
+			if (dwAttrId == SCARD_ATTR_DEVICE_FRIENDLY_NAME)
+			{
+				unsigned int len = strlen(rContext->lpcReader)+1;
+
+				*pcbAttrLen = len;
+				if (len > *pcbAttrLen)
+					rv = SCARD_E_INSUFFICIENT_BUFFER;
+				else
+				{
+					(void)strlcpy((char *)pbAttr, rContext->lpcReader,
+						*pcbAttrLen);
+					rv = SCARD_S_SUCCESS;
+				}
+
+			}
+			else 
+				rv = SCARD_E_UNSUPPORTED_FEATURE;
+			break;
+		case IFD_ERROR_INSUFFICIENT_BUFFER:
+			rv = SCARD_E_INSUFFICIENT_BUFFER;
+			break;
+		default:
+			rv = SCARD_E_NOT_TRANSACTED;
+	}
+
+	return rv;
 }
 
 LONG SCardSetAttrib(SCARDHANDLE hCard, DWORD dwAttrId,
 	LPCBYTE pbAttr, DWORD cbAttrLen)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 
 	if (0 == hCard)
 		return SCARD_E_INVALID_HANDLE;
@@ -1521,7 +1528,7 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 	LPDWORD pcbRecvLength)
 {
 	LONG rv;
-	PREADER_CONTEXT rContext = NULL;
+	READER_CONTEXT * rContext = NULL;
 	SCARD_IO_HEADER sSendPci, sRecvPci;
 	DWORD dwRxLength, tempRxLength;
 
@@ -1537,12 +1544,6 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 	if (pbSendBuffer == NULL || pbRecvBuffer == NULL || pioSendPci == NULL)
 		return SCARD_E_INVALID_PARAMETER;
 
-	/*
-	 * Must at least send a 4 bytes APDU
-	 */
-	if (cbSendLength < 4)
-		return SCARD_E_INVALID_PARAMETER;
-
 	/*
 	 * Must at least have 2 status words even for SCardControl
 	 */
@@ -1636,7 +1637,8 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 
 	tempRxLength = dwRxLength;
 
-	if (pioSendPci->dwProtocol == SCARD_PROTOCOL_RAW)
+	if ((pioSendPci->dwProtocol == SCARD_PROTOCOL_RAW)
+		&& (rContext->version == IFD_HVERSION_2_0))
 	{
 		rv = IFDControl_v2(rContext, (PUCHAR) pbSendBuffer, cbSendLength,
 			pbRecvBuffer, &dwRxLength);
@@ -1657,7 +1659,7 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 	{
 		*pcbRecvLength = 0;
 		Log2(PCSC_LOG_ERROR, "Card not transacted: 0x%08lX", rv);
-		return SCARD_E_NOT_TRANSACTED;
+		return rv;
 	}
 
 	/*
@@ -1676,20 +1678,27 @@ LONG SCardTransmit(SCARDHANDLE hCard, LPCSCARD_IO_REQUEST pioSendPci,
 	return SCARD_S_SUCCESS;
 }
 
-LONG SCardListReaders(SCARDCONTEXT hContext, LPCSTR mszGroups,
-	LPSTR mszReaders, LPDWORD pcchReaders)
+LONG SCardListReaders(/*@unused@*/ SCARDCONTEXT hContext,
+	/*@unused@*/ LPCSTR mszGroups,
+	/*@unused@*/ LPSTR mszReaders,
+	/*@unused@*/ LPDWORD pcchReaders)
 {
 	/*
 	 * Client side function
 	 */
+	(void)hContext;
+	(void)mszGroups;
+	(void)mszReaders;
+	(void)pcchReaders;
 	return SCARD_S_SUCCESS;
 }
 
-LONG SCardCancel(SCARDCONTEXT hContext)
+LONG SCardCancel(/*@unused@*/ SCARDCONTEXT hContext)
 {
 	/*
 	 * Client side function
 	 */
+	(void)hContext;
 	return SCARD_S_SUCCESS;
 }
 
