@@ -3,10 +3,12 @@
  *
  * Copyright (C) 2001-2004
  *  David Corcoran <corcoran@linuxnet.com>
+ * Copyright (C) 2003-2004
  *  Damien Sauveron <damien.sauveron@labri.fr>
- *  Ludoic Rousseau <ludovic.rousseau@free.fr>
+ * Copyright (C) 2002-2010
+ *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: winscard_msg.c 3087 2008-08-11 13:40:53Z rousseau $
+ * $Id: winscard_msg.c 4952 2010-05-18 13:42:25Z rousseau $
  */
 
 /**
@@ -38,27 +40,11 @@
 #include "misc.h"
 #include "pcscd.h"
 #include "winscard.h"
-#include "debug.h"
+#include "debuglog.h"
 #include "winscard_msg.h"
 #include "sys_generic.h"
 #include "utils.h"
 
-/**
- * @brief Wrapper for the SHMMessageReceive() function.
- *
- * Called by clients to read the server responses.
- *
- * @param[out] msgStruct Message read.
- * @param[in] dwClientID Client socket handle.
- * @param[in] blockamount Timeout in milliseconds.
- *
- * @return Same error codes as SHMMessageReceive().
- */
-INTERNAL int32_t SHMClientRead(psharedSegmentMsg msgStruct, uint32_t dwClientID, int32_t blockamount)
-{
-	return SHMMessageReceive(msgStruct, sizeof(*msgStruct), dwClientID, blockamount);
-}
-
 /**
  * @brief Prepares a communication channel for the client to talk to the server.
  *
@@ -72,13 +58,13 @@ INTERNAL int32_t SHMClientRead(psharedSegmentMsg msgStruct, uint32_t dwClientID,
  * @retval -1 The socket can not open a connection.
  * @retval -1 Can not set the socket to non-blocking.
  */
-INTERNAL int SHMClientSetupSession(uint32_t *pdwClientID)
+INTERNAL int ClientSetupSession(uint32_t *pdwClientID)
 {
 	struct sockaddr_un svc_addr;
 	int one;
 	int ret;
 
-	ret = socket(AF_UNIX, SOCK_STREAM, 0);
+	ret = socket(PF_UNIX, SOCK_STREAM, 0);
 	if (ret < 0)
 	{
 		Log2(PCSC_LOG_CRITICAL, "Error: create on client socket: %s",
@@ -96,7 +82,7 @@ INTERNAL int SHMClientSetupSession(uint32_t *pdwClientID)
 	{
 		Log3(PCSC_LOG_CRITICAL, "Error: connect to client socket %s: %s",
 			PCSCLITE_CSOCK_NAME, strerror(errno));
-		SYS_CloseFile(*pdwClientID);
+		(void)close(*pdwClientID);
 		return -1;
 	}
 
@@ -105,7 +91,7 @@ INTERNAL int SHMClientSetupSession(uint32_t *pdwClientID)
 	{
 		Log3(PCSC_LOG_CRITICAL, "Error: cannot set socket %s nonblocking: %s",
 			PCSCLITE_CSOCK_NAME, strerror(errno));
-		SYS_CloseFile(*pdwClientID);
+		(void)close(*pdwClientID);
 		return -1;
 	}
 
@@ -119,10 +105,9 @@ INTERNAL int SHMClientSetupSession(uint32_t *pdwClientID)
  *
  * @retval 0 Success.
  */
-INTERNAL int SHMClientCloseSession(uint32_t dwClientID)
+INTERNAL int ClientCloseSession(uint32_t dwClientID)
 {
-	SYS_CloseFile(dwClientID);
-	return 0;
+	return close(dwClientID);
 }
 
 /**
@@ -133,92 +118,71 @@ INTERNAL int SHMClientCloseSession(uint32_t dwClientID)
  * @param[in] buffer_void Message to be sent.
  * @param[in] buffer_size Size of the message to send
  * @param[in] filedes Socket handle.
- * @param[in] blockAmount Timeout in milliseconds.
+ * @param[in] timeOut Timeout in milliseconds.
  *
  * @retval 0 Success
  * @retval -1 Timeout.
  * @retval -1 Socket is closed.
  * @retval -1 A signal was received.
  */
-INTERNAL int SHMMessageSend(void *buffer_void, uint64_t buffer_size,
-	int32_t filedes, int32_t blockAmount)
+INTERNAL int32_t MessageSend(void *buffer_void, uint64_t buffer_size,
+	int32_t filedes)
 {
 	char *buffer = buffer_void;
 
-	/*
-	 * default is success
-	 */
+	/* default is success */
 	int retval = 0;
-	/*
-	 * record the time when we started
-	 */
-	time_t start = time(0);
-	/*
-	 * how many bytes remains to be written
-	 */
+
+	/* how many bytes remains to be written */
 	size_t remaining = buffer_size;
 
-	/*
-	 * repeat until all data is written
-	 */
+	/* repeat until all data is written */
 	while (remaining > 0)
 	{
 		fd_set write_fd;
-		struct timeval timeout;
 		int selret;
 
 		FD_ZERO(&write_fd);
 		FD_SET(filedes, &write_fd);
 
-		timeout.tv_usec = 0;
-		if ((timeout.tv_sec = start + blockAmount - time(0)) < 0)
-		{
-			/*
-			 * we already timed out
-			 */
-			retval = -1;
-			break;
-		}
+		selret = select(filedes + 1, NULL, &write_fd, NULL, NULL);
 
-		selret = select(filedes + 1, NULL, &write_fd, NULL, &timeout);
-
-		/*
-		 * try to write only when the file descriptor is writable
-		 */
+		/* try to write only when the file descriptor is writable */
 		if (selret > 0)
 		{
 			int written;
 
 			if (!FD_ISSET(filedes, &write_fd))
 			{
-				/*
-				 * very strange situation. it should be an assert really
-				 */
+				/* very strange situation. it should be an assert really */
 				retval = -1;
 				break;
 			}
+			/* since we are a user library we can't play with signals
+			 * The signals may already be used by the application */
+#ifdef MSG_NOSIGNAL
+			/* Get EPIPE return code instead of SIGPIPE signal
+			 * Works on Linux */
+			written = send(filedes, buffer, remaining, MSG_NOSIGNAL);
+#else
+			/* we may get a SIGPIPE signal if the other side has closed */
 			written = write(filedes, buffer, remaining);
+#endif
 
 			if (written > 0)
 			{
-				/*
-				 * we wrote something
-				 */
+				/* we wrote something */
 				buffer += written;
 				remaining -= written;
 			} else if (written == 0)
 			{
-				/*
-				 * peer closed the socket
-				 */
+				/* peer closed the socket */
 				retval = -1;
 				break;
 			} else
 			{
-				/*
-				 * we ignore the signals and socket full situations, all
-				 * other errors are fatal
-				 */
+				/* we ignore the signals and socket full situations, all
+				 * other errors are fatal */ 
 				if (errno != EINTR && errno != EAGAIN)
 				{
 					retval = -1;
@@ -227,16 +191,12 @@ INTERNAL int SHMMessageSend(void *buffer_void, uint64_t buffer_size,
 			}
 		} else if (selret == 0)
 		{
-			/*
-			 * timeout
-			 */
+			/* timeout */
 			retval = -1;
 			break;
 		} else
 		{
-			/*
-			 * ignore signals
-			 */
+			/* ignore signals */
 			if (errno != EINTR)
 			{
 				Log2(PCSC_LOG_ERROR, "select returns with failure: %s",
@@ -258,67 +218,41 @@ INTERNAL int SHMMessageSend(void *buffer_void, uint64_t buffer_size,
  * @param[out] buffer_void Message read.
  * @param[in] buffer_size Size to read
  * @param[in] filedes Socket handle.
- * @param[in] blockAmount Timeout in milliseconds.
  *
  * @retval 0 Success.
- * @retval -1 Timeout.
  * @retval -1 Socket is closed.
  * @retval -1 A signal was received.
  */
-INTERNAL int SHMMessageReceive(void *buffer_void, uint64_t buffer_size,
-	int32_t filedes, int32_t blockAmount)
+INTERNAL int32_t MessageReceive(void *buffer_void, uint64_t buffer_size,
+	int32_t filedes)
 {
 	char *buffer = buffer_void;
 
-	/*
-	 * default is success
-	 */
+	/* default is success */
 	int retval = 0;
-	/*
-	 * record the time when we started
-	 */
-	time_t start = time(0);
-	/*
-	 * how many bytes we must read
-	 */
+
+	/* how many bytes we must read */
 	size_t remaining = buffer_size;
 
-	/*
-	 * repeat until we get the whole message
-	 */
+	/* repeat until we get the whole message */
 	while (remaining > 0)
 	{
 		fd_set read_fd;
-		struct timeval timeout;
 		int selret;
 
 		FD_ZERO(&read_fd);
 		FD_SET(filedes, &read_fd);
 
-		timeout.tv_usec = 0;
-		if ((timeout.tv_sec = start + blockAmount - time(0)) < 0)
-		{
-			/*
-			 * we already timed out
-			 */
-			retval = -1;
-			break;
-		}
+		selret = select(filedes + 1, &read_fd, NULL, NULL, NULL);
 
-		selret = select(filedes + 1, &read_fd, NULL, NULL, &timeout);
-
-		/*
-		 * try to read only when socket is readable
-		 */
+		/* try to read only when socket is readable */
 		if (selret > 0)
 		{
 			int readed;
 
 			if (!FD_ISSET(filedes, &read_fd))
 			{
-				/*
-				 * very strange situation. it should be an assert really
-				 */
+				/* very strange situation. it should be an assert really */
 				retval = -1;
 				break;
 			}
@@ -326,61 +260,28 @@ INTERNAL int SHMMessageReceive(void *buffer_void, uint64_t buffer_size,
 
 			if (readed > 0)
 			{
-				/*
-				 * we got something
-				 */
+				/* we got something */
 				buffer += readed;
 				remaining -= readed;
 			} else if (readed == 0)
 			{
-				/*
-				 * peer closed the socket
-				 */
+				/* peer closed the socket */
 				retval = -1;
 				break;
 			} else
 			{
-				/*
-				 * we ignore the signals and empty socket situations, all
-				 * other errors are fatal
-				 */
+				/* we ignore the signals and empty socket situations, all
+				 * other errors are fatal */
 				if (errno != EINTR && errno != EAGAIN)
 				{
 					retval = -1;
 					break;
 				}
 			}
-		} else if (selret == 0)
+		}
+		else
 		{
-#ifdef PCSCD
-			/*
-			 * timeout
-			 */
-			retval = -1;
-			break;
-#else
-			/* is the daemon still there? */
-			if (SCardCheckDaemonAvailability() != SCARD_S_SUCCESS)
-			{
-				/*
-				 * timeout
-				 */
-				retval = -1;
-				break;
-			}
-
-			/* the command is extra slow so we keep waiting */
-			start = time(0);
-
-			/* you need to set the env variable PCSCLITE_DEBUG=0 since this is
-			 * logged on the client side and not on the pcscd side*/
-			Log1(PCSC_LOG_INFO, "Command not yet finished");
-#endif
-		} else
-		{
-			/*
-			 * we ignore signals, all other errors are fatal
-			 */
+			/* we ignore signals, all other errors are fatal */
 			if (errno != EINTR)
 			{
 				Log2(PCSC_LOG_ERROR, "select returns with failure: %s",
@@ -395,7 +296,139 @@ INTERNAL int SHMMessageReceive(void *buffer_void, uint64_t buffer_size,
 }
 
 /**
- * @brief Wrapper for the SHMMessageSend() function.
+ * @brief Called by the Client to get the reponse from the server or vice-versa.
+ *
+ * Reads the message from the file \c filedes.
+ *
+ * @param[in] command one of the \ref pcsc_msg_commands commands
+ * @param[out] buffer_void Message read.
+ * @param[in] buffer_size Size to read
+ * @param[in] filedes Socket handle.
+ * @param[in] timeOut Timeout in milliseconds.
+ *
+ * @retval 0 Success.
+ * @retval -1 Timeout.
+ * @retval -1 Socket is closed.
+ * @retval -1 A signal was received.
+ */
+INTERNAL int32_t MessageReceiveTimeout(uint32_t command, void *buffer_void,
+	uint64_t buffer_size, int32_t filedes, int32_t timeOut)
+{
+	char *buffer = buffer_void;
+
+	/* default is success */
+	int retval = 0;
+
+	/* record the time when we started */
+	struct timeval start;
+
+	/* how many bytes we must read */
+	size_t remaining = buffer_size;
+
+	gettimeofday(&start, NULL);
+
+	/* repeat until we get the whole message */
+	while (remaining > 0)
+	{
+		fd_set read_fd;
+		struct timeval timeout, now;
+		int selret;
+		long delta;
+
+		gettimeofday(&now, NULL);
+		delta = time_sub(&now, &start);
+
+		if (delta > timeOut*1000)
+		{
+			/* we already timed out */
+			retval = -2;
+			break;
+		}
+
+		/* remaining time to wait */
+		delta = timeOut*1000 - delta;
+
+		FD_ZERO(&read_fd);
+		FD_SET(filedes, &read_fd);
+
+		timeout.tv_sec = delta/1000000;
+		timeout.tv_usec = delta - timeout.tv_sec*1000000;
+
+		selret = select(filedes + 1, &read_fd, NULL, NULL, &timeout);
+
+		/* try to read only when socket is readable */
+		if (selret > 0)
+		{
+			int readed;
+
+			if (!FD_ISSET(filedes, &read_fd))
+			{
+				/* very strange situation. it should be an assert really */
+				retval = -1;
+				break;
+			}
+			readed = read(filedes, buffer, remaining);
+
+			if (readed > 0)
+			{
+				/* we got something */
+				buffer += readed;
+				remaining -= readed;
+			} else if (readed == 0)
+			{
+				/* peer closed the socket */
+				retval = -1;
+				break;
+			} else
+			{
+				/* we ignore the signals and empty socket situations, all
+				 * other errors are fatal */
+				if (errno != EINTR && errno != EAGAIN)
+				{
+					retval = -1;
+					break;
+				}
+			}
+		} else if (selret == 0)
+		{
+#ifdef PCSCD
+			(void)command;
+
+			/* timeout */
+			retval = -1;
+			break;
+#else
+			/* is the daemon still there? */
+			if (SCardCheckDaemonAvailability() != SCARD_S_SUCCESS)
+			{
+				/* timeout */
+				retval = -1;
+				break;
+			}
+
+			/* you need to set the env variable PCSCLITE_DEBUG=0 since
+			 * this is logged on the client side and not on the pcscd
+			 * side*/
+			Log2(PCSC_LOG_INFO, "Command 0x%X not yet finished", command);
+#endif
+		} else
+		{
+			/* we ignore signals, all other errors are fatal */
+			if (errno != EINTR)
+			{
+				Log2(PCSC_LOG_ERROR, "select returns with failure: %s",
+					strerror(errno));
+				retval = -1;
+				break;
+			}
+		}
+	}
+
+	return retval;
+}
+
+/**
+ * @brief Wrapper for the MessageSend() function.
  *
  * Called by clients to send messages to the server.
  * The parameters \p command and \p data are set in the \c sharedSegmentMsg
@@ -404,64 +437,24 @@ INTERNAL int SHMMessageReceive(void *buffer_void, uint64_t buffer_size,
  * @param[in] command Command to be sent.
  * @param[in] dwClientID Client socket handle.
  * @param[in] size Size of the message (\p data).
- * @param[in] blockAmount Timeout to the operation in ms.
+ * @param[in] timeOut Timeout to the operation in ms.
  * @param[in] data_void Data to be sent.
  *
- * @return Same error codes as SHMMessageSend().
+ * @return Same error codes as MessageSend().
  */
-INTERNAL int32_t WrapSHMWrite(uint32_t command, uint32_t dwClientID,
-	uint64_t size, uint32_t blockAmount, void *data_void)
+INTERNAL int32_t MessageSendWithHeader(uint32_t command, uint32_t dwClientID,
+	uint64_t size, void *data_void)
 {
-	char *data = data_void;
-
-	sharedSegmentMsg msgStruct;
+	struct rxHeader header;
 	int ret;
 
-	/*
-	 * Set the appropriate packet parameters
-	 */
+	/* header */
+	header.command = command;
+	header.size = size;
+	ret = MessageSend(&header, sizeof(header), dwClientID);
 
-	memset(&msgStruct, 0, sizeof(msgStruct));
-	msgStruct.mtype = CMD_FUNCTION;
-	msgStruct.user_id = SYS_GetUID();
-	msgStruct.group_id = SYS_GetGID();
-	msgStruct.command = command;
-	msgStruct.date = time(NULL);
-	memset(msgStruct.key, 0, sizeof(msgStruct.key));
-	if ((SCARD_TRANSMIT_EXTENDED == command)
-		|| (SCARD_CONTROL_EXTENDED == command))
-	{
-		/* first block */
-		if (size > sizeof(msgStruct.data))
-			memcpy(msgStruct.data, data, sizeof(msgStruct.data));
-		else
-		{
-			memcpy(msgStruct.data, data, size);
-			memset(msgStruct.data+size, 0, sizeof(msgStruct.data)-size);
-		}
-
-		ret = SHMMessageSend(&msgStruct, sizeof(msgStruct), dwClientID,
-			blockAmount);
-
-		/* do not send an empty second block */
-		if ((0 == ret) && (size > sizeof(msgStruct.data)))
-		{
-			/* second block */
-			ret = SHMMessageSend(data+sizeof(msgStruct.data),
-				size-sizeof(msgStruct.data), dwClientID, blockAmount);
-		}
-	}
-	else
-	{
-		memcpy(msgStruct.data, data, size);
-
-		ret = SHMMessageSend(&msgStruct, sizeof(msgStruct), dwClientID,
-			blockAmount);
-	}
-
-	if (SCARD_TRANSMIT == command)
-		/* clean APDU buffer to remove any possible PIN or secret value */
-		memset(msgStruct.data, 0, min(size, sizeof(msgStruct.data)));
+	/* command */
+	ret = MessageSend(data_void, size, dwClientID);
 
 	return ret;
 }
@@ -475,9 +468,9 @@ INTERNAL int32_t WrapSHMWrite(uint32_t command, uint32_t dwClientID,
  * @param[in] sockValue Socket to be closed.
  * @param[in] pcFilePath File used by the socket.
  */
-INTERNAL void SHMCleanupSharedSegment(int sockValue, const char *pcFilePath)
+INTERNAL void CleanupSharedSegment(int sockValue, const char *pcFilePath)
 {
-	SYS_CloseFile(sockValue);
-	SYS_RemoveFile(pcFilePath);
+	(void)close(sockValue);
+	(void)remove(pcFilePath);
 }
 
